@@ -6,7 +6,6 @@
 
 #include "TransportSecurityInfo.h"
 
-#include "DateTimeFormat.h"
 #include "PSMRunnable.h"
 #include "ipc/IPCMessageUtils.h"
 #include "mozilla/Casting.h"
@@ -40,14 +39,7 @@ namespace mozilla {
 namespace psm {
 
 TransportSecurityInfo::TransportSecurityInfo()
-    : mCipherSuite(0),
-      mProtocolVersion(0),
-      mCertificateTransparencyStatus(
-          nsITransportSecurityInfo::CERTIFICATE_TRANSPARENCY_NOT_APPLICABLE),
-      mKeaGroup(),
-      mSignatureSchemeName(),
-      mIsDelegatedCredential(false),
-      mIsDomainMismatch(false),
+    : mIsDomainMismatch(false),
       mIsNotValidAtThisTime(false),
       mIsUntrusted(false),
       mIsEV(false),
@@ -56,6 +48,14 @@ TransportSecurityInfo::TransportSecurityInfo()
       mHaveCertErrorBits(false),
       mCanceled(false),
       mMutex("TransportSecurityInfo::mMutex"),
+      mCipherSuite(0),
+      mProtocolVersion(0),
+      mCertificateTransparencyStatus(
+          nsITransportSecurityInfo::CERTIFICATE_TRANSPARENCY_NOT_APPLICABLE),
+      mKeaGroup(),
+      mSignatureSchemeName(),
+      mIsAcceptedEch(false),
+      mIsDelegatedCredential(false),
       mNPNCompleted(false),
       mResumed(false),
       mIsBuiltCertChainRootBuiltInRoot(false),
@@ -66,7 +66,22 @@ TransportSecurityInfo::TransportSecurityInfo()
 NS_IMPL_ISUPPORTS(TransportSecurityInfo, nsITransportSecurityInfo,
                   nsIInterfaceRequestor, nsISerializable, nsIClassInfo)
 
+void TransportSecurityInfo::SetPreliminaryHandshakeInfo(
+    const SSLChannelInfo& channelInfo, const SSLCipherSuiteInfo& cipherInfo) {
+  MutexAutoLock lock(mMutex);
+  mResumed = channelInfo.resumed;
+  mCipherSuite = channelInfo.cipherSuite;
+  mProtocolVersion = channelInfo.protocolVersion & 0xFF;
+  mKeaGroup.Assign(getKeaGroupName(channelInfo.keaGroup));
+  mSignatureSchemeName.Assign(getSignatureName(channelInfo.signatureScheme));
+  mIsDelegatedCredential = channelInfo.peerDelegCred;
+  mIsAcceptedEch = channelInfo.echAccepted;
+  mHaveCipherSuiteAndProtocol = true;
+}
+
 void TransportSecurityInfo::SetHostName(const char* host) {
+  MutexAutoLock lock(mMutex);
+
   mHostName.Assign(host);
 }
 
@@ -74,6 +89,8 @@ void TransportSecurityInfo::SetPort(int32_t aPort) { mPort = aPort; }
 
 void TransportSecurityInfo::SetOriginAttributes(
     const OriginAttributes& aOriginAttributes) {
+  MutexAutoLock lock(mMutex);
+
   mOriginAttributes = aOriginAttributes;
 }
 
@@ -81,8 +98,6 @@ void TransportSecurityInfo::SetOriginAttributes(
 // case, this returns (by pointer) 0, which is treated as a successful value.
 NS_IMETHODIMP
 TransportSecurityInfo::GetErrorCode(int32_t* state) {
-  MutexAutoLock lock(mMutex);
-
   // We're in an inconsistent state if we think we've been canceled but no error
   // code was set or we haven't been canceled but an error code was set.
   MOZ_ASSERT(
@@ -102,7 +117,6 @@ void TransportSecurityInfo::SetCanceled(PRErrorCode errorCode) {
     errorCode = SEC_ERROR_LIBRARY_FAILURE;
   }
 
-  MutexAutoLock lock(mMutex);
   mErrorCode = errorCode;
   mCanceled = true;
 }
@@ -121,8 +135,6 @@ void TransportSecurityInfo::SetSecurityState(uint32_t aState) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetErrorCodeString(nsAString& aErrorString) {
-  MutexAutoLock lock(mMutex);
-
   const char* codeName = PR_ErrorToName(mErrorCode);
   aErrorString.Truncate();
   if (codeName) {
@@ -138,6 +150,7 @@ TransportSecurityInfo::GetInterface(const nsIID& uuid, void** result) {
     NS_ERROR("nsNSSSocketInfo::GetInterface called off the main thread");
     return NS_ERROR_NOT_SAME_THREAD;
   }
+  MutexAutoLock lock(mMutex);
 
   nsresult rv;
   if (!mCallbacks) {
@@ -194,7 +207,7 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* aStream) {
   // Re-purpose mErrorMessageCached to represent serialization version
   // If string doesn't match exact version it will be treated as older
   // serialization.
-  rv = aStream->WriteWStringZ(NS_ConvertUTF8toUTF16("5").get());
+  rv = aStream->WriteWStringZ(NS_ConvertUTF8toUTF16("7").get());
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -275,6 +288,16 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* aStream) {
     return rv;
   }
 
+  rv = aStream->WriteBoolean(mIsAcceptedEch);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  rv = aStream->WriteStringZ(mPeerId.get());
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
   return NS_OK;
 }
 
@@ -285,7 +308,10 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* aStream) {
 
 // This is for backward compatibility to be able to read nsISSLStatus
 // serialized object.
-nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream) {
+nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream,
+                                              MutexAutoLock& aProofOfLock) {
+  mMutex.AssertCurrentThreadOwns();
+
   bool nsISSLStatusPresent;
   nsresult rv = aStream->ReadBoolean(&nsISSLStatusPresent);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
@@ -345,26 +371,26 @@ nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream) {
   const uint8_t streamFormatVersion =
       (protocolVersionAndStreamFormatVersion >> 8) & 0xFF;
 
-  rv = aStream->ReadBoolean(&mIsDomainMismatch);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsDomainMismatch);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->ReadBoolean(&mIsNotValidAtThisTime);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsNotValidAtThisTime);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->ReadBoolean(&mIsUntrusted);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsUntrusted);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->ReadBoolean(&mIsEV);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsEV);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = aStream->ReadBoolean(&mHasIsEVStatus);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHasIsEVStatus);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->ReadBoolean(&mHaveCipherSuiteAndProtocol);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHaveCipherSuiteAndProtocol);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = aStream->ReadBoolean(&mHaveCertErrorBits);
+  rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHaveCertErrorBits);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -391,7 +417,7 @@ nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream) {
 
   // Added in version 3 (see bug 1406856).
   if (streamFormatVersion >= 3) {
-    rv = ReadCertList(aStream, mSucceededCertChain);
+    rv = ReadCertList(aStream, mSucceededCertChain, aProofOfLock);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     if (NS_FAILED(rv)) {
@@ -400,7 +426,7 @@ nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream) {
 
     // Read only to consume bytes from the stream.
     nsTArray<RefPtr<nsIX509Cert>> failedCertChain;
-    rv = ReadCertList(aStream, failedCertChain);
+    rv = ReadCertList(aStream, failedCertChain, aProofOfLock);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     if (NS_FAILED(rv)) {
@@ -413,7 +439,8 @@ nsresult TransportSecurityInfo::ReadSSLStatus(nsIObjectInputStream* aStream) {
 // This is for backward compatability to be able to read nsIX509CertList
 // serialized object.
 nsresult TransportSecurityInfo::ReadCertList(
-    nsIObjectInputStream* aStream, nsTArray<RefPtr<nsIX509Cert>>& aCertList) {
+    nsIObjectInputStream* aStream, nsTArray<RefPtr<nsIX509Cert>>& aCertList,
+    MutexAutoLock& aProofOfLock) {
   bool nsIX509CertListPresent;
 
   nsresult rv = aStream->ReadBoolean(&nsIX509CertListPresent);
@@ -449,12 +476,13 @@ nsresult TransportSecurityInfo::ReadCertList(
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return ReadCertificatesFromStream(aStream, certListSize, aCertList);
+  return ReadCertificatesFromStream(aStream, certListSize, aCertList,
+                                    aProofOfLock);
 }
 
 nsresult TransportSecurityInfo::ReadCertificatesFromStream(
     nsIObjectInputStream* aStream, uint32_t aSize,
-    nsTArray<RefPtr<nsIX509Cert>>& aCertList) {
+    nsTArray<RefPtr<nsIX509Cert>>& aCertList, MutexAutoLock& aProofOfLock) {
   nsresult rv;
   for (uint32_t i = 0; i < aSize; ++i) {
     nsCOMPtr<nsISupports> support;
@@ -489,7 +517,7 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
 
   MutexAutoLock lock(mMutex);
 
-  rv = aStream->Read32(&mSecurityState);
+  rv = ReadUint32AndSetAtomicFieldHelper(aStream, mSecurityState);
   CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv), "Deserialization should not fail");
   if (NS_FAILED(rv)) {
     return rv;
@@ -532,12 +560,25 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
     return rv;
   }
 
+  int32_t serVersionParsedToInt = 0;
+
+  if (!serVersion.IsEmpty()) {
+    char first = serVersion.First();
+    // Check whether the first character of serVersion is a number
+    // since ToInteger() skipps some non integer values.
+    if (first >= '0' && first <= '9') {
+      nsresult error = NS_OK;
+      serVersionParsedToInt = serVersion.ToInteger(&error);
+      if (NS_FAILED(error)) {
+        return error;
+      }
+    }
+  }
+
   // moved from nsISSLStatus
-  if (!serVersion.EqualsASCII("1") && !serVersion.EqualsASCII("2") &&
-      !serVersion.EqualsASCII("3") && !serVersion.EqualsASCII("4") &&
-      !serVersion.EqualsASCII("5")) {
+  if (serVersionParsedToInt < 1) {
     // nsISSLStatus may be present
-    rv = ReadSSLStatus(aStream);
+    rv = ReadSSLStatus(aStream, lock);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
@@ -566,32 +607,32 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = aStream->ReadBoolean(&mIsDomainMismatch);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsDomainMismatch);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = aStream->ReadBoolean(&mIsNotValidAtThisTime);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsNotValidAtThisTime);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = aStream->ReadBoolean(&mIsUntrusted);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsUntrusted);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = aStream->ReadBoolean(&mIsEV);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mIsEV);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = aStream->ReadBoolean(&mHasIsEVStatus);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHasIsEVStatus);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = aStream->ReadBoolean(&mHaveCipherSuiteAndProtocol);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHaveCipherSuiteAndProtocol);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = aStream->ReadBoolean(&mHaveCertErrorBits);
+    rv = ReadBoolAndSetAtomicFieldHelper(aStream, mHaveCertErrorBits);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
@@ -611,10 +652,9 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (!serVersion.EqualsASCII("3") && !serVersion.EqualsASCII("4") &&
-        !serVersion.EqualsASCII("5")) {
+    if (serVersionParsedToInt < 3) {
       // The old data structure of certList(nsIX509CertList) presents
-      rv = ReadCertList(aStream, mSucceededCertChain);
+      rv = ReadCertList(aStream, mSucceededCertChain, lock);
       CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                               "Deserialization should not fail");
       NS_ENSURE_SUCCESS(rv, rv);
@@ -625,15 +665,15 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
                               "Deserialization should not fail");
       NS_ENSURE_SUCCESS(rv, rv);
 
-      rv = ReadCertificatesFromStream(aStream, certCount, mSucceededCertChain);
+      rv = ReadCertificatesFromStream(aStream, certCount, mSucceededCertChain,
+                                      lock);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
   // END moved from nsISSLStatus
-  if (!serVersion.EqualsASCII("3") && !serVersion.EqualsASCII("4") &&
-      !serVersion.EqualsASCII("5")) {
+  if (serVersionParsedToInt < 3) {
     // The old data structure of certList(nsIX509CertList) presents
-    rv = ReadCertList(aStream, mFailedCertChain);
+    rv = ReadCertList(aStream, mFailedCertChain, lock);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
@@ -644,13 +684,12 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
                             "Deserialization should not fail");
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = ReadCertificatesFromStream(aStream, certCount, mFailedCertChain);
+    rv = ReadCertificatesFromStream(aStream, certCount, mFailedCertChain, lock);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
   // mIsDelegatedCredential added in bug 1562773
-  if (serVersion.EqualsASCII("2") || serVersion.EqualsASCII("3") ||
-      serVersion.EqualsASCII("4") || serVersion.EqualsASCII("5")) {
+  if (serVersionParsedToInt >= 2) {
     rv = aStream->ReadBoolean(&mIsDelegatedCredential);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
@@ -660,7 +699,7 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
   }
 
   // mNPNCompleted, mNegotiatedNPN, mResumed added in bug 1584104
-  if (serVersion.EqualsASCII("4") || serVersion.EqualsASCII("5")) {
+  if (serVersionParsedToInt >= 4) {
     rv = aStream->ReadBoolean(&mNPNCompleted);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
@@ -684,7 +723,7 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
   }
 
   // mIsBuiltCertChainRootBuiltInRoot added in bug 1485652
-  if (serVersion.EqualsASCII("5")) {
+  if (serVersionParsedToInt >= 5) {
     rv = aStream->ReadBoolean(&mIsBuiltCertChainRootBuiltInRoot);
     CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
                             "Deserialization should not fail");
@@ -693,68 +732,88 @@ TransportSecurityInfo::Read(nsIObjectInputStream* aStream) {
     }
   }
 
+  // mIsAcceptedEch added in bug 1678079
+  if (serVersionParsedToInt >= 6) {
+    rv = aStream->ReadBoolean(&mIsAcceptedEch);
+    CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
+                            "Deserialization should not fail");
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
+  // mPeerId added in bug 1738664
+  if (serVersionParsedToInt >= 7) {
+    rv = aStream->ReadCString(mPeerId);
+    CHILD_DIAGNOSTIC_ASSERT(NS_SUCCEEDED(rv),
+                            "Deserialization should not fail");
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
   return NS_OK;
 }
 
 #undef CHILD_DIAGNOSTIC_ASSERT
 
-void TransportSecurityInfo::SerializeToIPC(IPC::Message* aMsg) {
+void TransportSecurityInfo::SerializeToIPC(IPC::MessageWriter* aWriter) {
   MutexAutoLock guard(mMutex);
 
   int32_t errorCode = static_cast<int32_t>(mErrorCode);
 
-  WriteParam(aMsg, mSecurityState);
-  WriteParam(aMsg, errorCode);
-  WriteParam(aMsg, mServerCert);
-  WriteParam(aMsg, mCipherSuite);
-  WriteParam(aMsg, mProtocolVersion);
-  WriteParam(aMsg, mIsDomainMismatch);
-  WriteParam(aMsg, mIsNotValidAtThisTime);
-  WriteParam(aMsg, mIsUntrusted);
-  WriteParam(aMsg, mIsEV);
-  WriteParam(aMsg, mHasIsEVStatus);
-  WriteParam(aMsg, mHaveCipherSuiteAndProtocol);
-  WriteParam(aMsg, mHaveCertErrorBits);
-  WriteParam(aMsg, mCertificateTransparencyStatus);
-  WriteParam(aMsg, mKeaGroup);
-  WriteParam(aMsg, mSignatureSchemeName);
-  WriteParam(aMsg, mSucceededCertChain);
-  WriteParam(aMsg, mFailedCertChain);
-  WriteParam(aMsg, mIsDelegatedCredential);
-  WriteParam(aMsg, mNPNCompleted);
-  WriteParam(aMsg, mNegotiatedNPN);
-  WriteParam(aMsg, mResumed);
-  WriteParam(aMsg, mIsBuiltCertChainRootBuiltInRoot);
+  WriteParam(aWriter, static_cast<uint32_t>(mSecurityState));
+  WriteParam(aWriter, errorCode);
+  WriteParam(aWriter, mServerCert);
+  WriteParam(aWriter, mCipherSuite);
+  WriteParam(aWriter, mProtocolVersion);
+  WriteParam(aWriter, static_cast<bool>(mIsDomainMismatch));
+  WriteParam(aWriter, static_cast<bool>(mIsNotValidAtThisTime));
+  WriteParam(aWriter, static_cast<bool>(mIsUntrusted));
+  WriteParam(aWriter, static_cast<bool>(mIsEV));
+  WriteParam(aWriter, static_cast<bool>(mHasIsEVStatus));
+  WriteParam(aWriter, static_cast<bool>(mHaveCipherSuiteAndProtocol));
+  WriteParam(aWriter, static_cast<bool>(mHaveCertErrorBits));
+  WriteParam(aWriter, mCertificateTransparencyStatus);
+  WriteParam(aWriter, mKeaGroup);
+  WriteParam(aWriter, mSignatureSchemeName);
+  WriteParam(aWriter, mSucceededCertChain);
+  WriteParam(aWriter, mFailedCertChain);
+  WriteParam(aWriter, mIsDelegatedCredential);
+  WriteParam(aWriter, mNPNCompleted);
+  WriteParam(aWriter, mNegotiatedNPN);
+  WriteParam(aWriter, mResumed);
+  WriteParam(aWriter, mIsBuiltCertChainRootBuiltInRoot);
+  WriteParam(aWriter, mIsAcceptedEch);
+  WriteParam(aWriter, mPeerId);
 }
 
-bool TransportSecurityInfo::DeserializeFromIPC(const IPC::Message* aMsg,
-                                               PickleIterator* aIter) {
+bool TransportSecurityInfo::DeserializeFromIPC(IPC::MessageReader* aReader) {
   MutexAutoLock guard(mMutex);
 
   int32_t errorCode = 0;
 
-  if (!ReadParam(aMsg, aIter, &mSecurityState) ||
-      !ReadParam(aMsg, aIter, &errorCode) ||
-      !ReadParam(aMsg, aIter, &mServerCert) ||
-      !ReadParam(aMsg, aIter, &mCipherSuite) ||
-      !ReadParam(aMsg, aIter, &mProtocolVersion) ||
-      !ReadParam(aMsg, aIter, &mIsDomainMismatch) ||
-      !ReadParam(aMsg, aIter, &mIsNotValidAtThisTime) ||
-      !ReadParam(aMsg, aIter, &mIsUntrusted) ||
-      !ReadParam(aMsg, aIter, &mIsEV) ||
-      !ReadParam(aMsg, aIter, &mHasIsEVStatus) ||
-      !ReadParam(aMsg, aIter, &mHaveCipherSuiteAndProtocol) ||
-      !ReadParam(aMsg, aIter, &mHaveCertErrorBits) ||
-      !ReadParam(aMsg, aIter, &mCertificateTransparencyStatus) ||
-      !ReadParam(aMsg, aIter, &mKeaGroup) ||
-      !ReadParam(aMsg, aIter, &mSignatureSchemeName) ||
-      !ReadParam(aMsg, aIter, &mSucceededCertChain) ||
-      !ReadParam(aMsg, aIter, &mFailedCertChain) ||
-      !ReadParam(aMsg, aIter, &mIsDelegatedCredential) ||
-      !ReadParam(aMsg, aIter, &mNPNCompleted) ||
-      !ReadParam(aMsg, aIter, &mNegotiatedNPN) ||
-      !ReadParam(aMsg, aIter, &mResumed) ||
-      !ReadParam(aMsg, aIter, &mIsBuiltCertChainRootBuiltInRoot)) {
+  if (!ReadParamAtomicHelper(aReader, mSecurityState) ||
+      !ReadParam(aReader, &errorCode) || !ReadParam(aReader, &mServerCert) ||
+      !ReadParam(aReader, &mCipherSuite) ||
+      !ReadParam(aReader, &mProtocolVersion) ||
+      !ReadParamAtomicHelper(aReader, mIsDomainMismatch) ||
+      !ReadParamAtomicHelper(aReader, mIsNotValidAtThisTime) ||
+      !ReadParamAtomicHelper(aReader, mIsUntrusted) ||
+      !ReadParamAtomicHelper(aReader, mIsEV) ||
+      !ReadParamAtomicHelper(aReader, mHasIsEVStatus) ||
+      !ReadParamAtomicHelper(aReader, mHaveCipherSuiteAndProtocol) ||
+      !ReadParamAtomicHelper(aReader, mHaveCertErrorBits) ||
+      !ReadParam(aReader, &mCertificateTransparencyStatus) ||
+      !ReadParam(aReader, &mKeaGroup) ||
+      !ReadParam(aReader, &mSignatureSchemeName) ||
+      !ReadParam(aReader, &mSucceededCertChain) ||
+      !ReadParam(aReader, &mFailedCertChain) ||
+      !ReadParam(aReader, &mIsDelegatedCredential) ||
+      !ReadParam(aReader, &mNPNCompleted) ||
+      !ReadParam(aReader, &mNegotiatedNPN) || !ReadParam(aReader, &mResumed) ||
+      !ReadParam(aReader, &mIsBuiltCertChainRootBuiltInRoot) ||
+      !ReadParam(aReader, &mIsAcceptedEch) || !ReadParam(aReader, &mPeerId)) {
     return false;
   }
 
@@ -855,7 +914,7 @@ void RememberCertErrorsTable::RememberCertHasError(
     bits.mIsUntrusted = infoObject->mIsUntrusted;
 
     MutexAutoLock lock(mMutex);
-    mErrorHosts.Put(hostPortKey, bits);
+    mErrorHosts.InsertOrUpdate(hostPortKey, bits);
   } else {
     MutexAutoLock lock(mMutex);
     mErrorHosts.Remove(hostPortKey);
@@ -895,10 +954,8 @@ void RememberCertErrorsTable::LookupCertErrorBits(
   infoObject->mIsUntrusted = bits.mIsUntrusted;
 }
 
-void TransportSecurityInfo::SetStatusErrorBits(nsNSSCertificate* cert,
-                                               uint32_t collected_errors) {
-  MutexAutoLock lock(mMutex);
-
+void TransportSecurityInfo::SetStatusErrorBits(
+    const nsCOMPtr<nsIX509Cert>& cert, uint32_t collected_errors) {
   SetServerCert(cert, EVStatus::NotEV);
 
   mHaveCertErrorBits = true;
@@ -913,6 +970,7 @@ NS_IMETHODIMP
 TransportSecurityInfo::GetFailedCertChain(
     nsTArray<RefPtr<nsIX509Cert>>& aFailedCertChain) {
   MOZ_ASSERT(aFailedCertChain.IsEmpty());
+  MutexAutoLock lock(mMutex);
   if (!aFailedCertChain.IsEmpty()) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -925,11 +983,7 @@ static nsresult CreateCertChain(nsTArray<RefPtr<nsIX509Cert>>& aOutput,
   nsTArray<nsTArray<uint8_t>> certList = std::move(aCertList);
   aOutput.Clear();
   for (auto& certBytes : certList) {
-    RefPtr<nsIX509Cert> cert = nsNSSCertificate::ConstructFromDER(
-        BitwiseCast<char*, uint8_t*>(certBytes.Elements()), certBytes.Length());
-    if (!cert) {
-      return NS_ERROR_FAILURE;
-    }
+    RefPtr<nsIX509Cert> cert = new nsNSSCertificate(std::move(certBytes));
     aOutput.AppendElement(cert);
   }
   return NS_OK;
@@ -937,20 +991,24 @@ static nsresult CreateCertChain(nsTArray<RefPtr<nsIX509Cert>>& aOutput,
 
 nsresult TransportSecurityInfo::SetFailedCertChain(
     nsTArray<nsTArray<uint8_t>>&& aCertList) {
+  MutexAutoLock lock(mMutex);
+
   return CreateCertChain(mFailedCertChain, std::move(aCertList));
 }
 
 NS_IMETHODIMP TransportSecurityInfo::GetServerCert(nsIX509Cert** aServerCert) {
   NS_ENSURE_ARG_POINTER(aServerCert);
+  MutexAutoLock lock(mMutex);
 
   nsCOMPtr<nsIX509Cert> cert = mServerCert;
   cert.forget(aServerCert);
   return NS_OK;
 }
 
-void TransportSecurityInfo::SetServerCert(nsNSSCertificate* aServerCert,
-                                          EVStatus aEVStatus) {
+void TransportSecurityInfo::SetServerCert(
+    const nsCOMPtr<nsIX509Cert>& aServerCert, EVStatus aEVStatus) {
   MOZ_ASSERT(aServerCert);
+  MutexAutoLock lock(mMutex);
 
   mServerCert = aServerCert;
   mIsEV = (aEVStatus == EVStatus::EV);
@@ -961,6 +1019,7 @@ NS_IMETHODIMP
 TransportSecurityInfo::GetSucceededCertChain(
     nsTArray<RefPtr<nsIX509Cert>>& aSucceededCertChain) {
   MOZ_ASSERT(aSucceededCertChain.IsEmpty());
+  MutexAutoLock lock(mMutex);
   if (!aSucceededCertChain.IsEmpty()) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -970,11 +1029,13 @@ TransportSecurityInfo::GetSucceededCertChain(
 
 nsresult TransportSecurityInfo::SetSucceededCertChain(
     nsTArray<nsTArray<uint8_t>>&& aCertList) {
+  MutexAutoLock lock(mMutex);
   return CreateCertChain(mSucceededCertChain, std::move(aCertList));
 }
 
 NS_IMETHODIMP TransportSecurityInfo::SetIsBuiltCertChainRootBuiltInRoot(
     bool aIsBuiltInRoot) {
+  MutexAutoLock lock(mMutex);
   mIsBuiltCertChainRootBuiltInRoot = aIsBuiltInRoot;
   return NS_OK;
 }
@@ -982,12 +1043,15 @@ NS_IMETHODIMP TransportSecurityInfo::SetIsBuiltCertChainRootBuiltInRoot(
 NS_IMETHODIMP TransportSecurityInfo::GetIsBuiltCertChainRootBuiltInRoot(
     bool* aIsBuiltInRoot) {
   NS_ENSURE_ARG_POINTER(aIsBuiltInRoot);
+  MutexAutoLock lock(mMutex);
   *aIsBuiltInRoot = mIsBuiltCertChainRootBuiltInRoot;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetCipherName(nsACString& aCipherName) {
+  MutexAutoLock lock(mMutex);
+
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1005,6 +1069,7 @@ TransportSecurityInfo::GetCipherName(nsACString& aCipherName) {
 NS_IMETHODIMP
 TransportSecurityInfo::GetKeyLength(uint32_t* aKeyLength) {
   NS_ENSURE_ARG_POINTER(aKeyLength);
+  MutexAutoLock lock(mMutex);
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1022,6 +1087,7 @@ TransportSecurityInfo::GetKeyLength(uint32_t* aKeyLength) {
 NS_IMETHODIMP
 TransportSecurityInfo::GetSecretKeyLength(uint32_t* aSecretKeyLength) {
   NS_ENSURE_ARG_POINTER(aSecretKeyLength);
+  MutexAutoLock lock(mMutex);
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1038,6 +1104,8 @@ TransportSecurityInfo::GetSecretKeyLength(uint32_t* aSecretKeyLength) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetKeaGroupName(nsACString& aKeaGroup) {
+  MutexAutoLock lock(mMutex);
+
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1048,6 +1116,8 @@ TransportSecurityInfo::GetKeaGroupName(nsACString& aKeaGroup) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetSignatureSchemeName(nsACString& aSignatureScheme) {
+  MutexAutoLock lock(mMutex);
+
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1058,6 +1128,8 @@ TransportSecurityInfo::GetSignatureSchemeName(nsACString& aSignatureScheme) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetProtocolVersion(uint16_t* aProtocolVersion) {
+  MutexAutoLock lock(mMutex);
+
   NS_ENSURE_ARG_POINTER(aProtocolVersion);
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
@@ -1071,6 +1143,7 @@ NS_IMETHODIMP
 TransportSecurityInfo::GetCertificateTransparencyStatus(
     uint16_t* aCertificateTransparencyStatus) {
   NS_ENSURE_ARG_POINTER(aCertificateTransparencyStatus);
+  MutexAutoLock lock(mMutex);
 
   *aCertificateTransparencyStatus = mCertificateTransparencyStatus;
   return NS_OK;
@@ -1104,23 +1177,9 @@ uint16_t TransportSecurityInfo::ConvertCertificateTransparencyInfoToStatus(
   return nsITransportSecurityInfo::CERTIFICATE_TRANSPARENCY_NOT_APPLICABLE;
 }
 
-// static
-nsTArray<nsTArray<uint8_t>> TransportSecurityInfo::CreateCertBytesArray(
-    const UniqueCERTCertList& aCertChain) {
-  nsTArray<nsTArray<uint8_t>> certsBytes;
-  for (CERTCertListNode* n = CERT_LIST_HEAD(aCertChain);
-       !CERT_LIST_END(n, aCertChain); n = CERT_LIST_NEXT(n)) {
-    nsTArray<uint8_t> certBytes;
-    certBytes.AppendElements(n->cert->derCert.data, n->cert->derCert.len);
-    certsBytes.AppendElement(std::move(certBytes));
-  }
-  return certsBytes;
-}
-
 NS_IMETHODIMP
 TransportSecurityInfo::GetIsDomainMismatch(bool* aIsDomainMismatch) {
   NS_ENSURE_ARG_POINTER(aIsDomainMismatch);
-
   *aIsDomainMismatch = mHaveCertErrorBits && mIsDomainMismatch;
   return NS_OK;
 }
@@ -1128,7 +1187,6 @@ TransportSecurityInfo::GetIsDomainMismatch(bool* aIsDomainMismatch) {
 NS_IMETHODIMP
 TransportSecurityInfo::GetIsNotValidAtThisTime(bool* aIsNotValidAtThisTime) {
   NS_ENSURE_ARG_POINTER(aIsNotValidAtThisTime);
-
   *aIsNotValidAtThisTime = mHaveCertErrorBits && mIsNotValidAtThisTime;
   return NS_OK;
 }
@@ -1136,7 +1194,6 @@ TransportSecurityInfo::GetIsNotValidAtThisTime(bool* aIsNotValidAtThisTime) {
 NS_IMETHODIMP
 TransportSecurityInfo::GetIsUntrusted(bool* aIsUntrusted) {
   NS_ENSURE_ARG_POINTER(aIsUntrusted);
-
   *aIsUntrusted = mHaveCertErrorBits && mIsUntrusted;
   return NS_OK;
 }
@@ -1145,7 +1202,6 @@ NS_IMETHODIMP
 TransportSecurityInfo::GetIsExtendedValidation(bool* aIsEV) {
   NS_ENSURE_ARG_POINTER(aIsEV);
   *aIsEV = false;
-
   // Never allow bad certs for EV, regardless of overrides.
   if (mHaveCertErrorBits) {
     return NS_OK;
@@ -1159,9 +1215,21 @@ TransportSecurityInfo::GetIsExtendedValidation(bool* aIsEV) {
   return NS_ERROR_NOT_AVAILABLE;
 }
 
+NS_IMETHODIMP
+TransportSecurityInfo::GetIsAcceptedEch(bool* aIsAcceptedEch) {
+  NS_ENSURE_ARG_POINTER(aIsAcceptedEch);
+  MutexAutoLock lock(mMutex);
+  if (!mHaveCipherSuiteAndProtocol) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  *aIsAcceptedEch = mIsAcceptedEch;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 TransportSecurityInfo::GetIsDelegatedCredential(bool* aIsDelegCred) {
   NS_ENSURE_ARG_POINTER(aIsDelegCred);
+  MutexAutoLock lock(mMutex);
   if (!mHaveCipherSuiteAndProtocol) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -1171,6 +1239,8 @@ TransportSecurityInfo::GetIsDelegatedCredential(bool* aIsDelegCred) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetNegotiatedNPN(nsACString& aNegotiatedNPN) {
+  MutexAutoLock lock(mMutex);
+
   if (!mNPNCompleted) {
     return NS_ERROR_NOT_CONNECTED;
   }
@@ -1181,6 +1251,8 @@ TransportSecurityInfo::GetNegotiatedNPN(nsACString& aNegotiatedNPN) {
 
 NS_IMETHODIMP
 TransportSecurityInfo::GetResumed(bool* aResumed) {
+  MutexAutoLock lock(mMutex);
+  NS_ENSURE_ARG_POINTER(aResumed);
   *aResumed = mResumed;
   return NS_OK;
 }
@@ -1190,5 +1262,12 @@ void TransportSecurityInfo::SetResumed(bool aResumed) {
   mResumed = aResumed;
 }
 
+NS_IMETHODIMP
+TransportSecurityInfo::GetPeerId(nsACString& aResult) {
+  MutexAutoLock lock(mMutex);
+  aResult.Assign(mPeerId);
+  return NS_OK;
+}
+
 }  // namespace psm
 }  // namespace mozilla
