@@ -3,15 +3,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "TextEventDispatcher.h"
+
+#include "IMEData.h"
+#include "PuppetWidget.h"
+#include "TextEvents.h"
+
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_dom.h"
-#include "mozilla/TextEvents.h"
-#include "mozilla/TextEventDispatcher.h"
 #include "nsIFrame.h"
 #include "nsIWidget.h"
 #include "nsPIDOMWindow.h"
 #include "nsView.h"
-#include "PuppetWidget.h"
 
 namespace mozilla {
 namespace widget {
@@ -227,8 +230,9 @@ void TextEventDispatcher::InitEvent(WidgetGUIEvent& aEvent) const {
 #endif  // #ifdef DEBUG
 }
 
-Maybe<WritingMode> TextEventDispatcher::MaybeWritingModeAtSelection() const {
-  if (mWritingMode.isSome()) {
+Maybe<WritingMode> TextEventDispatcher::MaybeQueryWritingModeAtSelection()
+    const {
+  if (mHasFocus || mWritingMode.isSome()) {
     return mWritingMode;
   }
 
@@ -236,21 +240,25 @@ Maybe<WritingMode> TextEventDispatcher::MaybeWritingModeAtSelection() const {
     return Nothing();
   }
 
+  // If a remote content has focus and IME does not have focus, it's going to
+  // fail eQuerySelectedText in ContentCacheParent.  For avoiding to waste
+  // unnecessary runtime cost and to prevent unnecessary warnings, we should
+  // not dispatch the event in the case.
+  const InputContext inputContext = mWidget->GetInputContext();
+  if (XRE_IsE10sParentProcess() && inputContext.IsOriginContentProcess() &&
+      !inputContext.mIMEState.IsEditable()) {
+    return Nothing();
+  }
+
   WidgetQueryContentEvent querySelectedTextEvent(true, eQuerySelectedText,
                                                  mWidget);
   nsEventStatus status = nsEventStatus_eIgnore;
   const_cast<TextEventDispatcher*>(this)->DispatchEvent(
       mWidget, querySelectedTextEvent, status);
   if (!querySelectedTextEvent.FoundSelection()) {
-    if (mHasFocus) {
-      mWritingMode.reset();
-    }
     return Nothing();
   }
 
-  if (mHasFocus) {
-    mWritingMode = Some(querySelectedTextEvent.mReply->mWritingMode);
-  }
   return Some(querySelectedTextEvent.mReply->mWritingMode);
 }
 
@@ -414,6 +422,10 @@ nsresult TextEventDispatcher::NotifyIME(
   nsresult rv = NS_ERROR_NOT_IMPLEMENTED;
 
   switch (aIMENotification.mMessage) {
+    case NOTIFY_IME_OF_FOCUS: {
+      mWritingMode = MaybeQueryWritingModeAtSelection();
+      break;
+    }
     case NOTIFY_IME_OF_BLUR:
       mHasFocus = false;
       mWritingMode.reset();
@@ -430,7 +442,7 @@ nsresult TextEventDispatcher::NotifyIME(
       }
       break;
     case NOTIFY_IME_OF_SELECTION_CHANGE:
-      if (mHasFocus) {
+      if (mHasFocus && aIMENotification.mSelectionChangeData.HasRange()) {
         mWritingMode =
             Some(aIMENotification.mSelectionChangeData.GetWritingMode());
       }
@@ -616,7 +628,7 @@ bool TextEventDispatcher::DispatchKeyboardEventInternal(
                              !aIndexOfKeypress);
     MOZ_DIAGNOSTIC_ASSERT_IF(
         aMessage == eKeyPress,
-        aIndexOfKeypress < std::max(keyEvent.mKeyValue.Length(), 1u));
+        aIndexOfKeypress < std::max<size_t>(keyEvent.mKeyValue.Length(), 1));
     char16_t ch =
         keyEvent.mKeyValue.IsEmpty() ? 0 : keyEvent.mKeyValue[aIndexOfKeypress];
     keyEvent.SetCharCode(static_cast<uint32_t>(ch));
@@ -644,9 +656,8 @@ bool TextEventDispatcher::DispatchKeyboardEventInternal(
     keyEvent.mNativeKeyEvent = aKeyboardEvent.mNativeKeyEvent;
   } else {
     // If it's not a keyboard event for native key event, we should ensure that
-    // mNativeKeyEvent and mPluginEvent are null/empty.
+    // mNativeKeyEvent is null.
     keyEvent.mNativeKeyEvent = nullptr;
-    keyEvent.mPluginEvent.Clear();
   }
   // TODO: Manage mUniqueId here.
 
@@ -694,6 +705,14 @@ bool TextEventDispatcher::DispatchKeyboardEventInternal(
     keyEvent.mFlags.mOnlySystemGroupDispatchInContent = true;
   }
 
+  // If an editable element has focus and we're in the parent process, we should
+  // retrieve native key bindings right now because even if it matches with a
+  // reserved shortcut key, it should be handled by the editor.
+  if (XRE_IsParentProcess() && mHasFocus &&
+      (aMessage == eKeyDown || aMessage == eKeyPress)) {
+    keyEvent.InitAllEditCommands(mWritingMode);
+  }
+
   DispatchInputEvent(mWidget, keyEvent, aStatus);
   return true;
 }
