@@ -5,34 +5,31 @@
 
 #include <algorithm>
 
+#include "GeckoViewStreamListener.h"
+#include "InetAddress.h"  // for java::sdk::InetAddress and java::sdk::UnknownHostException
+#include "ReferrerInfo.h"
 #include "WebExecutorSupport.h"
 
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsIHttpChannel.h"
 #include "nsIHttpChannelInternal.h"
 #include "nsIHttpHeaderVisitor.h"
-#include "nsINSSErrorsService.h"
-#include "nsIUploadChannel2.h"
-#include "nsIX509Cert.h"
-
 #include "nsIDNSService.h"
 #include "nsIDNSListener.h"
 #include "nsIDNSRecord.h"
+#include "nsINSSErrorsService.h"
+#include "nsNetUtil.h"  // for NS_NewURI, NS_NewChannel, NS_NewStreamLoader
+#include "nsIPrivateBrowsingChannel.h"
+#include "nsIUploadChannel2.h"
+#include "nsIX509Cert.h"
 
+#include "mozilla/Preferences.h"
+#include "mozilla/net/CookieJarSettings.h"
+#include "mozilla/net/DNS.h"  // for NetAddr
 #include "mozilla/java/GeckoWebExecutorWrappers.h"
 #include "mozilla/java/WebMessageWrappers.h"
 #include "mozilla/java/WebRequestErrorWrappers.h"
 #include "mozilla/java/WebResponseWrappers.h"
-#include "mozilla/net/DNS.h"  // for NetAddr
-#include "mozilla/net/CookieJarSettings.h"
-#include "mozilla/Preferences.h"
-#include "GeckoViewStreamListener.h"
-#include "nsIPrivateBrowsingChannel.h"
-
-#include "nsNetUtil.h"  // for NS_NewURI, NS_NewChannel, NS_NewStreamLoader
-
-#include "InetAddress.h"  // for java::sdk::InetAddress and java::sdk::UnknownHostException
-#include "ReferrerInfo.h"
 
 namespace mozilla {
 using namespace net;
@@ -353,6 +350,11 @@ static nsresult SetupHttpChannel(nsIHttpChannel* aHttpChannel,
   rv = internalChannel->SetFetchCacheMode(cacheMode);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  if (req->BeConservative()) {
+    rv = internalChannel->SetBeConservative(true);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   // We don't have any UI
   rv = internalChannel->SetBlockAuthPrompt(true);
   NS_ENSURE_SUCCESS(rv, rv);
