@@ -10,15 +10,14 @@
 
 class nsDeviceContextSpecAndroid final : public nsIDeviceContextSpec {
  private:
-  ~nsDeviceContextSpecAndroid() {}
+  virtual ~nsDeviceContextSpecAndroid();
 
  public:
   NS_DECL_ISUPPORTS
 
   already_AddRefed<PrintTarget> MakePrintTarget() final;
 
-  NS_IMETHOD Init(nsIWidget* aWidget, nsIPrintSettings* aPS,
-                  bool aIsPrintPreview) override;
+  NS_IMETHOD Init(nsIPrintSettings* aPS, bool aIsPrintPreview) override;
   NS_IMETHOD BeginDocument(const nsAString& aTitle,
                            const nsAString& aPrintToFileName,
                            int32_t aStartPage, int32_t aEndPage) override;
@@ -27,7 +26,6 @@ class nsDeviceContextSpecAndroid final : public nsIDeviceContextSpec {
   NS_IMETHOD EndPage() override { return NS_OK; }
 
  private:
-  nsCOMPtr<nsIPrintSettings> mPrintSettings;
   nsCOMPtr<nsIFile> mTempFile;
 };
 #endif  // nsDeviceContextAndroid_h__
