@@ -26,14 +26,13 @@
 #include "nsRegion.h"
 #include "nsXULPopupManager.h"
 #include "nsMenuPopupFrame.h"
-#ifdef MOZ_XUL
-#  include "nsTreeBodyFrame.h"
-#endif
+#include "nsTreeBodyFrame.h"
 #include "mozilla/MouseEvents.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/SVGImageContext.h"
+#include "mozilla/TextControlElement.h"
 #include "mozilla/Unused.h"
 #include "mozilla/ViewportUtils.h"
 #include "mozilla/dom/BindingDeclarations.h"
@@ -64,6 +63,7 @@ nsBaseDragService::nsBaseDragService()
       mOnlyChromeDrop(false),
       mDoingDrag(false),
       mSessionIsSynthesizedForTests(false),
+      mIsDraggingTextInTextControl(false),
       mEndingSession(false),
       mHasImage(false),
       mUserCancelled(false),
@@ -129,16 +129,47 @@ nsBaseDragService::GetNumDropItems(uint32_t* aNumItems) {
 }
 
 //
-// GetSourceDocument
+// GetSourceWindowContext
 //
-// Returns the DOM document where the drag was initiated. This will be
+// Returns the window context where the drag was initiated. This will be
 // nullptr if the drag began outside of our application.
 //
 NS_IMETHODIMP
-nsBaseDragService::GetSourceDocument(Document** aSourceDocument) {
-  *aSourceDocument = mSourceDocument.get();
-  NS_IF_ADDREF(*aSourceDocument);
+nsBaseDragService::GetSourceWindowContext(
+    WindowContext** aSourceWindowContext) {
+  *aSourceWindowContext = mSourceWindowContext.get();
+  NS_IF_ADDREF(*aSourceWindowContext);
+  return NS_OK;
+}
 
+NS_IMETHODIMP
+nsBaseDragService::SetSourceWindowContext(WindowContext* aSourceWindowContext) {
+  // This should only be called in a child process.
+  MOZ_ASSERT(!XRE_IsParentProcess());
+  mSourceWindowContext = aSourceWindowContext;
+  return NS_OK;
+}
+
+//
+// GetSourceTopWindowContext
+//
+// Returns the top-level window context where the drag was initiated. This will
+// be nullptr if the drag began outside of our application.
+//
+NS_IMETHODIMP
+nsBaseDragService::GetSourceTopWindowContext(
+    WindowContext** aSourceTopWindowContext) {
+  *aSourceTopWindowContext = mSourceTopWindowContext.get();
+  NS_IF_ADDREF(*aSourceTopWindowContext);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsBaseDragService::SetSourceTopWindowContext(
+    WindowContext* aSourceTopWindowContext) {
+  // This should only be called in a child process.
+  MOZ_ASSERT(!XRE_IsParentProcess());
+  mSourceTopWindowContext = aSourceTopWindowContext;
   return NS_OK;
 }
 
@@ -154,6 +185,26 @@ nsBaseDragService::GetSourceNode(nsINode** aSourceNode) {
   return NS_OK;
 }
 
+void nsBaseDragService::UpdateSource(nsINode* aNewSourceNode,
+                                     Selection* aNewSelection) {
+  MOZ_ASSERT(mSourceNode);
+  MOZ_ASSERT(aNewSourceNode);
+  MOZ_ASSERT(mSourceNode->IsInNativeAnonymousSubtree() ||
+             aNewSourceNode->IsInNativeAnonymousSubtree());
+  MOZ_ASSERT(mSourceDocument == aNewSourceNode->OwnerDoc());
+  mSourceNode = aNewSourceNode;
+  // Don't set mSelection if the session was invoked without selection or
+  // making it becomes nullptr.  The latter occurs when the old frame is
+  // being destroyed.
+  if (mSelection && aNewSelection) {
+    // XXX If the dragging image is created once (e.g., at drag start), the
+    //     image won't be updated unless we notify `DrawDrag` callers.
+    //     However, it must be okay for now to keep using older image of
+    //     Selection.
+    mSelection = aNewSelection;
+  }
+}
+
 NS_IMETHODIMP
 nsBaseDragService::GetTriggeringPrincipal(nsIPrincipal** aPrincipal) {
   NS_IF_ADDREF(*aPrincipal = mTriggeringPrincipal);
@@ -217,6 +268,10 @@ bool nsBaseDragService::IsSynthesizedForTests() {
   return mSessionIsSynthesizedForTests;
 }
 
+bool nsBaseDragService::IsDraggingTextInTextControl() {
+  return mIsDraggingTextInTextControl;
+}
+
 uint32_t nsBaseDragService::GetEffectAllowedForTests() {
   MOZ_ASSERT(mSessionIsSynthesizedForTests);
   return mEffectAllowedForTests;
@@ -227,16 +282,22 @@ NS_IMETHODIMP nsBaseDragService::SetDragEndPointForTests(int32_t aScreenX,
   MOZ_ASSERT(mDoingDrag);
   MOZ_ASSERT(mSourceDocument);
   MOZ_ASSERT(mSessionIsSynthesizedForTests);
+
   if (!mDoingDrag || !mSourceDocument || !mSessionIsSynthesizedForTests) {
     return NS_ERROR_FAILURE;
   }
-  nsPresContext* presContext = mSourceDocument->GetPresContext();
-  if (NS_WARN_IF(!presContext)) {
+  nsPresContext* pc = mSourceDocument->GetPresContext();
+  if (NS_WARN_IF(!pc)) {
     return NS_ERROR_FAILURE;
   }
-  SetDragEndPoint(
-      LayoutDeviceIntPoint(presContext->CSSPixelsToDevPixels(aScreenX),
-                           presContext->CSSPixelsToDevPixels(aScreenY)));
+  auto p = LayoutDeviceIntPoint::Round(CSSIntPoint(aScreenX, aScreenY) *
+                                       pc->CSSToDevPixelScale());
+  // p is screen-relative, and we want them to be top-level-widget-relative.
+  if (nsCOMPtr<nsIWidget> widget = pc->GetRootWidget()) {
+    p -= widget->WidgetToScreenOffset();
+    p += widget->WidgetToTopLevelWidgetOffset();
+  }
+  SetDragEndPoint(p);
   return NS_OK;
 }
 
@@ -257,6 +318,10 @@ nsBaseDragService::InvokeDragSession(
   mTriggeringPrincipal = aPrincipal;
   mCsp = aCsp;
   mSourceNode = aDOMNode;
+  mIsDraggingTextInTextControl =
+      mSourceNode->IsInNativeAnonymousSubtree() &&
+      TextControlElement::FromNodeOrNull(
+          mSourceNode->GetClosestNativeAnonymousSubtreeRootParent());
   mContentPolicyType = aContentPolicyType;
   mEndDragPoint = LayoutDeviceIntPoint(0, 0);
 
@@ -349,9 +414,12 @@ nsBaseDragService::InvokeDragSessionWithImage(
   mImage = aImage;
   mImageOffset = CSSIntPoint(aImageX, aImageY);
   mDragStartData = nullptr;
+  mSourceWindowContext =
+      aDOMNode ? aDOMNode->OwnerDoc()->GetWindowContext() : nullptr;
+  mSourceTopWindowContext =
+      mSourceWindowContext ? mSourceWindowContext->TopWindowContext() : nullptr;
 
-  mScreenPosition.x = aDragEvent->ScreenX(CallerType::System);
-  mScreenPosition.y = aDragEvent->ScreenY(CallerType::System);
+  mScreenPosition = aDragEvent->ScreenPoint(CallerType::System);
   mInputSource = aDragEvent->MozInputSource();
 
   // If dragging within a XUL tree and no custom drag image was
@@ -360,7 +428,6 @@ nsBaseDragService::InvokeDragSessionWithImage(
   // tree to ensure that the drag feedback gets clipped to those
   // rows. For other content, region should be null.
   mRegion = Nothing();
-#ifdef MOZ_XUL
   if (aDOMNode && aDOMNode->IsContent() && !aImage) {
     if (aDOMNode->NodeInfo()->Equals(nsGkAtoms::treechildren,
                                      kNameSpaceID_XUL)) {
@@ -371,7 +438,6 @@ nsBaseDragService::InvokeDragSessionWithImage(
       }
     }
   }
-#endif
 
   nsresult rv = InvokeDragSession(
       aDOMNode, aPrincipal, aCsp, aCookieJarSettings, aTransferableArray,
@@ -399,9 +465,10 @@ nsBaseDragService::InvokeDragSessionWithRemoteImage(
   mImage = nullptr;
   mDragStartData = aDragStartData;
   mImageOffset = CSSIntPoint(0, 0);
+  mSourceWindowContext = mDragStartData->GetSourceWindowContext();
+  mSourceTopWindowContext = mDragStartData->GetSourceTopWindowContext();
 
-  mScreenPosition.x = aDragEvent->ScreenX(CallerType::System);
-  mScreenPosition.y = aDragEvent->ScreenY(CallerType::System);
+  mScreenPosition = aDragEvent->ScreenPoint(CallerType::System);
   mInputSource = aDragEvent->MozInputSource();
 
   nsresult rv = InvokeDragSession(
@@ -440,6 +507,9 @@ nsBaseDragService::InvokeDragSessionWithSelection(
   // XXXndeakin this should actually be the deepest node that contains both
   // endpoints of the selection
   nsCOMPtr<nsINode> node = aSelection->GetFocusNode();
+  mSourceWindowContext = node ? node->OwnerDoc()->GetWindowContext() : nullptr;
+  mSourceTopWindowContext =
+      mSourceWindowContext ? mSourceWindowContext->TopWindowContext() : nullptr;
 
   return InvokeDragSession(node, aPrincipal, aCsp, aCookieJarSettings,
                            aTransferableArray, aActionType,
@@ -547,6 +617,7 @@ nsBaseDragService::EndDragSession(bool aDoneDrag, uint32_t aKeyModifiers) {
 
   mDoingDrag = false;
   mSessionIsSynthesizedForTests = false;
+  mIsDraggingTextInTextControl = false;
   mEffectAllowedForTests = nsIDragService::DRAGDROP_ACTION_UNINITIALIZED;
   mEndingSession = false;
   mCanDrop = false;
@@ -554,6 +625,7 @@ nsBaseDragService::EndDragSession(bool aDoneDrag, uint32_t aKeyModifiers) {
   // release the source we've been holding on to.
   mSourceDocument = nullptr;
   mSourceNode = nullptr;
+  mSourceWindowContext = nullptr;
   mTriggeringPrincipal = nullptr;
   mCsp = nullptr;
   mSelection = nullptr;
@@ -599,32 +671,38 @@ void nsBaseDragService::DiscardInternalTransferData() {
 NS_IMETHODIMP
 nsBaseDragService::FireDragEventAtSource(EventMessage aEventMessage,
                                          uint32_t aKeyModifiers) {
-  if (mSourceNode && mSourceDocument && !mSuppressLevel) {
-    RefPtr<PresShell> presShell = mSourceDocument->GetPresShell();
-    if (presShell) {
-      nsEventStatus status = nsEventStatus_eIgnore;
-      WidgetDragEvent event(true, aEventMessage, nullptr);
-      event.mFlags.mIsSynthesizedForTests = mSessionIsSynthesizedForTests;
-      event.mInputSource = mInputSource;
-      if (aEventMessage == eDragEnd) {
-        event.mRefPoint = mEndDragPoint;
-        event.mUserCancelled = mUserCancelled;
-      }
-      event.mModifiers = aKeyModifiers;
-      // Send the drag event to APZ, which needs to know about them to be
-      // able to accurately detect the end of a drag gesture.
-      if (nsPresContext* presContext = presShell->GetPresContext()) {
-        if (nsCOMPtr<nsIWidget> widget = presContext->GetRootWidget()) {
-          widget->DispatchEventToAPZOnly(&event);
-        }
-      }
-
-      nsCOMPtr<nsIContent> content = do_QueryInterface(mSourceNode);
-      return presShell->HandleDOMEventWithTarget(content, &event, &status);
-    }
+  if (!mSourceNode || !mSourceDocument || mSuppressLevel) {
+    return NS_OK;
+  }
+  RefPtr<PresShell> presShell = mSourceDocument->GetPresShell();
+  if (!presShell) {
+    return NS_OK;
   }
 
-  return NS_OK;
+  RefPtr<nsPresContext> pc = presShell->GetPresContext();
+  nsCOMPtr<nsIWidget> widget = pc ? pc->GetRootWidget() : nullptr;
+
+  nsEventStatus status = nsEventStatus_eIgnore;
+  WidgetDragEvent event(true, aEventMessage, widget);
+  event.mFlags.mIsSynthesizedForTests = mSessionIsSynthesizedForTests;
+  event.mInputSource = mInputSource;
+  if (aEventMessage == eDragEnd) {
+    event.mRefPoint = mEndDragPoint;
+    if (widget) {
+      event.mRefPoint -= widget->WidgetToTopLevelWidgetOffset();
+    }
+    event.mUserCancelled = mUserCancelled;
+  }
+  event.mModifiers = aKeyModifiers;
+
+  if (widget) {
+    // Send the drag event to APZ, which needs to know about them to be
+    // able to accurately detect the end of a drag gesture.
+    widget->DispatchEventToAPZOnly(&event);
+  }
+
+  nsCOMPtr<nsIContent> content = do_QueryInterface(mSourceNode);
+  return presShell->HandleDOMEventWithTarget(content, &event, &status);
 }
 
 /* This is used by Windows and Mac to update the position of a popup being
@@ -640,7 +718,7 @@ nsBaseDragService::DragMoved(int32_t aX, int32_t aY) {
           RoundedToInt(LayoutDeviceIntPoint(aX, aY) /
                        frame->PresContext()->CSSToDevPixelScale()) -
           mImageOffset;
-      (static_cast<nsMenuPopupFrame*>(frame))->MoveTo(cssPos, true);
+      static_cast<nsMenuPopupFrame*>(frame)->MoveTo(cssPos, true);
     }
   }
 
@@ -701,11 +779,9 @@ nsresult nsBaseDragService::DrawDrag(nsINode* aDOMNode,
   }
 
   // convert mouse position to dev pixels of the prescontext
-  CSSIntPoint screenPosition(aScreenPosition);
-  screenPosition.x -= mImageOffset.x;
-  screenPosition.y -= mImageOffset.y;
-  LayoutDeviceIntPoint screenPoint =
-      ConvertToUnscaledDevPixels(*aPresContext, screenPosition);
+  const CSSIntPoint screenPosition = aScreenPosition - mImageOffset;
+  const auto screenPoint = LayoutDeviceIntPoint::Round(
+      screenPosition * (*aPresContext)->CSSToDevPixelScale());
   aScreenDragRect->MoveTo(screenPoint.x, screenPoint.y);
 
   // check if drag images are disabled
@@ -879,15 +955,6 @@ nsresult nsBaseDragService::DrawDragForImage(
   return result;
 }
 
-LayoutDeviceIntPoint nsBaseDragService::ConvertToUnscaledDevPixels(
-    nsPresContext* aPresContext, CSSIntPoint aScreenPosition) {
-  int32_t adj =
-      aPresContext->DeviceContext()->AppUnitsPerDevPixelAtUnitFullZoom();
-  return LayoutDeviceIntPoint(
-      nsPresContext::CSSPixelsToAppUnits(aScreenPosition.x) / adj,
-      nsPresContext::CSSPixelsToAppUnits(aScreenPosition.y) / adj);
-}
-
 NS_IMETHODIMP
 nsBaseDragService::Suppress() {
   EndDragSession(false, 0);
