@@ -6,33 +6,41 @@
 #ifndef nsIWidget_h__
 #define nsIWidget_h__
 
-#include "mozilla/UniquePtr.h"
-#include "nsISupports.h"
-#include "nsColor.h"
-#include "nsRect.h"
-#include "nsString.h"
-
-#include "nsCOMPtr.h"
-#include "nsWidgetInitData.h"
-#include "nsTArray.h"
-#include "nsITheme.h"
-#include "nsITimer.h"
-#include "nsRegionFwd.h"
-#include "nsXULAppAPI.h"
-#include "mozilla/Maybe.h"
+#include <cmath>
+#include <cstdint>
+#include "ErrorList.h"
+#include "Units.h"
+#include "mozilla/AlreadyAddRefed.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/EventForwards.h"
-#include "mozilla/layers/ScrollableLayerGuid.h"
-#include "mozilla/layers/ZoomConstraints.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/TimeStamp.h"
-#include "mozilla/gfx/Point.h"
+#include "mozilla/UniquePtr.h"
+#include "mozilla/gfx/Matrix.h"
+#include "mozilla/gfx/Rect.h"
+#include "mozilla/layers/LayersTypes.h"
+#include "mozilla/layers/ScrollableLayerGuid.h"
+#include "mozilla/layers/ZoomConstraints.h"
+#include "mozilla/image/Resolution.h"
 #include "mozilla/widget/IMEData.h"
-#include "VsyncSource.h"
-#include "nsDataHashtable.h"
+#include "nsCOMPtr.h"
+#include "nsColor.h"
+#include "nsDebug.h"
+#include "nsID.h"
 #include "nsIObserver.h"
+#include "nsISupports.h"
+#include "nsITheme.h"
+#include "nsITimer.h"
 #include "nsIWidgetListener.h"
-#include "Units.h"
-#include "mozilla/dom/BindingDeclarations.h"
+#include "nsRect.h"
+#include "nsSize.h"
+#include "nsStringFwd.h"
+#include "nsTArray.h"
+#include "nsTHashMap.h"
+#include "nsWidgetInitData.h"
+#include "nsXULAppAPI.h"
 
 // forward declarations
 class nsIBidiKeyboard;
@@ -42,11 +50,17 @@ class nsIContent;
 class ViewWrapper;
 class nsIScreen;
 class nsIRunnable;
-class nsIKeyEventInPluginCallback;
 
 namespace mozilla {
+enum class NativeKeyBindingsType : uint8_t;
+class VsyncDispatcher;
+class WidgetGUIEvent;
+class WidgetInputEvent;
+class WidgetKeyboardEvent;
+struct FontRange;
 
 enum class StyleWindowShadow : uint8_t;
+enum class ColorScheme : uint8_t;
 
 #if defined(MOZ_WIDGET_ANDROID)
 namespace ipc {
@@ -55,24 +69,17 @@ class Shmem;
 #endif  // defined(MOZ_WIDGET_ANDROID)
 namespace dom {
 class BrowserChild;
+enum class CallerType : uint32_t;
 }  // namespace dom
-namespace plugins {
-class PluginWidgetChild;
-}  // namespace plugins
+class WindowRenderer;
 namespace layers {
 class AsyncDragMetrics;
 class Compositor;
 class CompositorBridgeChild;
 struct FrameMetrics;
 class LayerManager;
-class LayerManagerComposite;
-class PLayerTransactionChild;
 class WebRenderBridgeChild;
 }  // namespace layers
-namespace gfx {
-class DrawTarget;
-class SourceSurface;
-}  // namespace gfx
 namespace widget {
 class TextEventDispatcher;
 class TextEventDispatcherListener;
@@ -114,16 +121,10 @@ typedef void* nsNativeWidget;
 #define NS_NATIVE_REGION 5
 #define NS_NATIVE_OFFSETX 6
 #define NS_NATIVE_OFFSETY 7
-#define NS_NATIVE_PLUGIN_PORT 8
 #define NS_NATIVE_SCREEN 9
 // The toplevel GtkWidget containing this nsIWidget:
 #define NS_NATIVE_SHELLWIDGET 10
-// Has to match to NPNVnetscapeWindow, and shareable across processes
-// HWND on Windows and XID on X11
-#define NS_NATIVE_SHAREABLE_WINDOW 11
 #define NS_NATIVE_OPENGL_CONTEXT 12
-// See RegisterPluginWindowForRemoteUpdates
-#define NS_NATIVE_PLUGIN_ID 13
 // This is available only with GetNativeData() in parent process.  Anybody
 // shouldn't access this pointer as a valid pointer since the result may be
 // special value like NS_ONLY_ONE_NATIVE_IME_CONTEXT.  So, the result is just
@@ -132,32 +133,23 @@ typedef void* nsNativeWidget;
 // XP code should use nsIWidget::GetNativeIMEContext() instead of using this.
 #define NS_RAW_NATIVE_IME_CONTEXT 14
 #define NS_NATIVE_WINDOW_WEBRTC_DEVICE_ID 15
-#ifdef XP_MACOSX
-#  define NS_NATIVE_PLUGIN_PORT_QD 100
-#  define NS_NATIVE_PLUGIN_PORT_CG 101
-#endif
 #ifdef XP_WIN
 #  define NS_NATIVE_TSF_THREAD_MGR 100
 #  define NS_NATIVE_TSF_CATEGORY_MGR 101
 #  define NS_NATIVE_TSF_DISPLAY_ATTR_MGR 102
 #  define NS_NATIVE_ICOREWINDOW 103  // winrt specific
-#  define NS_NATIVE_CHILD_WINDOW 104
-#  define NS_NATIVE_CHILD_OF_SHAREABLE_WINDOW 105
 #endif
 #if defined(MOZ_WIDGET_GTK)
-// set/get nsPluginNativeWindowGtk, e10s specific
-#  define NS_NATIVE_PLUGIN_OBJECT_PTR 104
-#  ifdef MOZ_X11
-#    define NS_NATIVE_COMPOSITOR_DISPLAY 105
-#  endif  // MOZ_X11
 #  define NS_NATIVE_EGL_WINDOW 106
 #endif
 #ifdef MOZ_WIDGET_ANDROID
 #  define NS_JAVA_SURFACE 100
-#  define NS_PRESENTATION_WINDOW 101
-#  define NS_PRESENTATION_SURFACE 102
+#  define NS_JAVA_SURFACE_CONTROL 101
 #endif
 
+#define MOZ_WIDGET_MAX_SIZE 16384
+#define MOZ_WIDGET_INVALID_SCALE 0.0
+
 // Must be kept in sync with xpcom/rust/xpcom/src/interfaces/nonidl.rs
 #define NS_IWIDGET_IID                               \
   {                                                  \
@@ -278,14 +270,34 @@ namespace widget {
  * Values are in device pixels.
  */
 struct SizeConstraints {
-  SizeConstraints() : mMaxSize(NS_MAXSIZE, NS_MAXSIZE) {}
+  SizeConstraints()
+      : mMaxSize(MOZ_WIDGET_MAX_SIZE, MOZ_WIDGET_MAX_SIZE),
+        mScale(MOZ_WIDGET_INVALID_SCALE) {}
 
   SizeConstraints(mozilla::LayoutDeviceIntSize aMinSize,
-                  mozilla::LayoutDeviceIntSize aMaxSize)
-      : mMinSize(aMinSize), mMaxSize(aMaxSize) {}
+                  mozilla::LayoutDeviceIntSize aMaxSize,
+                  mozilla::DesktopToLayoutDeviceScale aScale)
+      : mMinSize(aMinSize), mMaxSize(aMaxSize), mScale(aScale) {
+    if (mMaxSize.width > MOZ_WIDGET_MAX_SIZE) {
+      mMaxSize.width = MOZ_WIDGET_MAX_SIZE;
+    }
+    if (mMaxSize.height > MOZ_WIDGET_MAX_SIZE) {
+      mMaxSize.height = MOZ_WIDGET_MAX_SIZE;
+    }
+  }
 
   mozilla::LayoutDeviceIntSize mMinSize;
   mozilla::LayoutDeviceIntSize mMaxSize;
+
+  /*
+   * The scale used to convert from desktop to device dimensions.
+   * MOZ_WIDGET_INVALID_SCALE if the value is not known.
+   *
+   * Bug 1701109 is filed to revisit adding of 'mScale' and deal
+   * with multi-monitor scaling issues in more complete way across
+   * all widget implementations.
+   */
+  mozilla::DesktopToLayoutDeviceScale mScale;
 };
 
 struct AutoObserverNotifier {
@@ -299,7 +311,7 @@ struct AutoObserverNotifier {
       return 0;
     }
     uint64_t observerId = ++sObserverId;
-    sSavedObservers.Put(observerId, mObserver);
+    sSavedObservers.InsertOrUpdate(observerId, mObserver);
     SkipNotification();
     return observerId;
   }
@@ -329,8 +341,7 @@ struct AutoObserverNotifier {
 
  private:
   static uint64_t sObserverId;
-  static nsDataHashtable<nsUint64HashKey, nsCOMPtr<nsIObserver>>
-      sSavedObservers;
+  static nsTHashMap<uint64_t, nsCOMPtr<nsIObserver>> sSavedObservers;
 };
 
 }  // namespace widget
@@ -349,11 +360,12 @@ class nsIWidget : public nsISupports {
   typedef mozilla::layers::AsyncDragMetrics AsyncDragMetrics;
   typedef mozilla::layers::FrameMetrics FrameMetrics;
   typedef mozilla::layers::LayerManager LayerManager;
-  typedef mozilla::layers::LayerManagerComposite LayerManagerComposite;
+  typedef mozilla::WindowRenderer WindowRenderer;
   typedef mozilla::layers::LayersBackend LayersBackend;
-  typedef mozilla::layers::PLayerTransactionChild PLayerTransactionChild;
+  typedef mozilla::layers::LayersId LayersId;
   typedef mozilla::layers::ScrollableLayerGuid ScrollableLayerGuid;
   typedef mozilla::layers::ZoomConstraints ZoomConstraints;
+  typedef mozilla::widget::IMEEnabled IMEEnabled;
   typedef mozilla::widget::IMEMessage IMEMessage;
   typedef mozilla::widget::IMENotification IMENotification;
   typedef mozilla::widget::IMENotificationRequests IMENotificationRequests;
@@ -382,6 +394,13 @@ class nsIWidget : public nsISupports {
   typedef mozilla::CSSPoint CSSPoint;
   typedef mozilla::CSSRect CSSRect;
 
+  enum class WindowButtonType {
+    Minimize,
+    Maximize,
+    Close,
+    Count,
+  };
+
   // Used in UpdateThemeGeometries.
   struct ThemeGeometry {
     // The ThemeGeometryType value for the themed widget, see
@@ -502,17 +521,22 @@ class nsIWidget : public nsISupports {
    * nsView in connection with AttachViewToTopLevel above.
    */
   virtual void SetAttachedWidgetListener(nsIWidgetListener* aListener) = 0;
-  virtual nsIWidgetListener* GetAttachedWidgetListener() = 0;
+  virtual nsIWidgetListener* GetAttachedWidgetListener() const = 0;
   virtual void SetPreviouslyAttachedWidgetListener(
       nsIWidgetListener* aListener) = 0;
   virtual nsIWidgetListener* GetPreviouslyAttachedWidgetListener() = 0;
 
+  /**
+   * Notifies the root widget of a non-blank paint.
+   */
+  virtual void DidGetNonBlankPaint() {}
+
   /**
    * Accessor functions to get and set the listener which handles various
    * actions for the widget.
    */
   //@{
-  virtual nsIWidgetListener* GetWidgetListener() = 0;
+  virtual nsIWidgetListener* GetWidgetListener() const = 0;
   virtual void SetWidgetListener(nsIWidgetListener* alistener) = 0;
   //@}
 
@@ -557,7 +581,7 @@ class nsIWidget : public nsISupports {
   /**
    * Return the top (non-sheet) parent of this Widget if it's a sheet,
    * or nullptr if this isn't a sheet (or some other error occurred).
-   * Sheets are only supported on some platforms (currently only OS X).
+   * Sheets are only supported on some platforms (currently only macOS).
    *
    * @return the top (non-sheet) parent widget or nullptr
    *
@@ -696,8 +720,8 @@ class nsIWidget : public nsISupports {
    * following Move and Resize widget APIs.
    *
    * The display-/device-pixel distinction becomes important for (at least)
-   * Mac OS X with Hi-DPI (retina) displays, and Windows when the UI scale
-   * factor is set to other than 100%.
+   * macOS with Hi-DPI (retina) displays, and Windows when the UI scale factor
+   * is set to other than 100%.
    *
    * The Move and Resize methods take floating-point parameters, rather than
    * integer ones. This is important when manipulating top-level widgets,
@@ -923,6 +947,11 @@ class nsIWidget : public nsISupports {
    */
   virtual nsresult SetNonClientMargins(LayoutDeviceIntMargin& aMargins) = 0;
 
+  /**
+   * Sets the region around the edges of the window that can be dragged to
+   * resize the window. All four sides of the window will get the same margin.
+   */
+  virtual void SetResizeMargin(mozilla::LayoutDeviceIntCoord aResizeMargin) = 0;
   /**
    * Get the client offset from the window origin.
    *
@@ -939,16 +968,6 @@ class nsIWidget : public nsISupports {
     return GetClientBounds().Size();
   }
 
-  /**
-   * Get the size of the bounds of this widget that will be visible when
-   * rendered.
-   *
-   * @return the width and height of the composition size of this widget.
-   */
-  virtual LayoutDeviceIntSize GetCompositionSize() {
-    return GetBounds().Size();
-  }
-
   /**
    * Set the background color for this widget
    *
@@ -965,31 +984,41 @@ class nsIWidget : public nsISupports {
 
   virtual void ClearCachedCursor() = 0;
 
+  struct Cursor {
+    // The system cursor chosen by the page. This is used if there's no custom
+    // cursor, or if we fail to use the custom cursor in some way (if the image
+    // fails to load, for example).
+    nsCursor mDefaultCursor = eCursor_standard;
+    // May be null, to represent no custom cursor image.
+    nsCOMPtr<imgIContainer> mContainer;
+    uint32_t mHotspotX = 0;
+    uint32_t mHotspotY = 0;
+    mozilla::ImageResolution mResolution;
+
+    bool IsCustom() const { return !!mContainer; }
+
+    bool operator==(const Cursor& aOther) const {
+      return mDefaultCursor == aOther.mDefaultCursor &&
+             mContainer.get() == aOther.mContainer.get() &&
+             mHotspotX == aOther.mHotspotX && mHotspotY == aOther.mHotspotY &&
+             mResolution == aOther.mResolution;
+    }
+
+    bool operator!=(const Cursor& aOther) const { return !(*this == aOther); }
+  };
+
   /**
-   * Sets the cursor cursor for this widget.
-   *
-   * @param aDefaultCursor the default cursor to be set
-   * @param aCursorImage a custom cursor, maybe null.
-   * @param aX the X coordinate of the hotspot for aCursorImage (from left).
-   * @param aY the Y coordinate of the hotspot for aCursorImage (from top).
+   * Sets the cursor for this widget.
    */
-  virtual void SetCursor(nsCursor aDefaultCursor, imgIContainer* aCursorImage,
-                         uint32_t aHotspotX, uint32_t aHotspotY) = 0;
+  virtual void SetCursor(const Cursor&) = 0;
+
+  static nsIntSize CustomCursorSize(const Cursor&);
 
   /**
    * Get the window type of this widget.
    */
   nsWindowType WindowType() { return mWindowType; }
 
-  /**
-   * Determines if this widget is one of the three types of plugin widgets.
-   */
-  bool IsPlugin() {
-    return mWindowType == eWindowType_plugin ||
-           mWindowType == eWindowType_plugin_ipc_chrome ||
-           mWindowType == eWindowType_plugin_ipc_content;
-  }
-
   /**
    * Set the transparency mode of the top-level window containing this widget.
    * So, e.g., if you call this on the widget for an IFRAME, the top level
@@ -1015,94 +1044,6 @@ class nsIWidget : public nsISupports {
    */
   virtual nsTransparencyMode GetTransparencyMode() = 0;
 
-  /**
-   * This represents a command to set the bounds and clip region of
-   * a child widget.
-   */
-  struct Configuration {
-    nsCOMPtr<nsIWidget> mChild;
-    uintptr_t mWindowID;  // e10s specific, the unique plugin port id
-    bool mVisible;        // e10s specific, widget visibility
-    LayoutDeviceIntRect mBounds;
-    CopyableTArray<LayoutDeviceIntRect> mClipRegion;
-  };
-
-  /**
-   * Sets the clip region of each mChild (which must actually be a child
-   * of this widget) to the union of the pixel rects given in
-   * mClipRegion, all relative to the top-left of the child
-   * widget. Clip regions are not implemented on all platforms and only
-   * need to actually work for children that are plugins.
-   *
-   * Also sets the bounds of each child to mBounds.
-   *
-   * This will invalidate areas of the children that have changed, but
-   * does not need to invalidate any part of this widget.
-   *
-   * Children should be moved in the order given; the array is
-   * sorted so to minimize unnecessary invalidation if children are
-   * moved in that order.
-   */
-  virtual nsresult ConfigureChildren(
-      const nsTArray<Configuration>& aConfigurations) = 0;
-  virtual nsresult SetWindowClipRegion(
-      const nsTArray<LayoutDeviceIntRect>& aRects,
-      bool aIntersectWithExisting) = 0;
-
-  /**
-   * Appends to aRects the rectangles constituting this widget's clip
-   * region. If this widget is not clipped, appends a single rectangle
-   * (0, 0, bounds.width, bounds.height).
-   */
-  virtual void GetWindowClipRegion(nsTArray<LayoutDeviceIntRect>* aRects) = 0;
-
-  /**
-   * Register or unregister native plugin widgets which receive Configuration
-   * data from the content process via the compositor.
-   *
-   * Lookups are used by the main thread via the compositor to lookup widgets
-   * based on a unique window id. On Windows and Linux this is the
-   * NS_NATIVE_PLUGIN_PORT (hwnd/XID). This tracking maintains a reference to
-   * widgets held. Consumers are responsible for removing widgets from this
-   * list.
-   */
-  virtual void RegisterPluginWindowForRemoteUpdates() = 0;
-  virtual void UnregisterPluginWindowForRemoteUpdates() = 0;
-  static nsIWidget* LookupRegisteredPluginWindow(uintptr_t aWindowID);
-
-  /**
-   * Iterates across the list of registered plugin widgets and updates thier
-   * visibility based on which plugins are included in the 'visible' list.
-   *
-   * The compositor knows little about tabs, but it does know which plugin
-   * widgets are currently included in the visible layer tree. It calls this
-   * helper to hide widgets it knows nothing about.
-   */
-  static void UpdateRegisteredPluginWindowVisibility(
-      uintptr_t aOwnerWidget, nsTArray<uintptr_t>& aPluginIds);
-
-#if defined(XP_WIN)
-  /**
-   * Iterates over the list of registered plugins and for any that are owned
-   * by aOwnerWidget and visible it takes a snapshot.
-   *
-   * @param aOwnerWidget only captures visible widgets owned by this
-   */
-  static void CaptureRegisteredPlugins(uintptr_t aOwnerWidget);
-
-  /**
-   * Take a scroll capture for this widget if possible.
-   */
-  virtual void UpdateScrollCapture() = 0;
-
-  /**
-   * Creates an async ImageContainer to hold scroll capture images that can be
-   * used if the plugin is hidden during scroll.
-   * @return the async container ID of the created ImageContainer.
-   */
-  virtual uint64_t CreateScrollCaptureContainer() = 0;
-#endif
-
   /**
    * Set the shadow style of the window.
    *
@@ -1126,6 +1067,12 @@ class nsIWidget : public nsISupports {
    */
   virtual void SetWindowTransform(const mozilla::gfx::Matrix& aTransform) {}
 
+  /**
+   * Set the preferred color-scheme for the widget.
+   * Ignored on non-Mac platforms.
+   */
+  virtual void SetColorScheme(const mozilla::Maybe<mozilla::ColorScheme>&) {}
+
   /**
    * Set whether the window should ignore mouse events or not.
    *
@@ -1134,7 +1081,7 @@ class nsIWidget : public nsISupports {
   virtual void SetWindowMouseTransparent(bool aIsTransparent) {}
 
   /*
-   * On Mac OS X, this method shows or hides the pill button in the titlebar
+   * On macOS, this method shows or hides the pill button in the titlebar
    * that's used to collapse the toolbar.
    *
    * Ignored on child widgets and on non-Mac platforms.
@@ -1158,8 +1105,8 @@ class nsIWidget : public nsISupports {
   };
 
   /**
-   * Sets the kind of top-level window animation this widget should have.  On
-   * Mac OS X, this causes a particular kind of animation to be shown when the
+   * Sets the kind of top-level window animation this widget should have. On
+   * macOS, this causes a particular kind of animation to be shown when the
    * window is first made visible.
    *
    * Ignored on child widgets and on non-Mac platforms.
@@ -1169,16 +1116,10 @@ class nsIWidget : public nsISupports {
   /**
    * Specifies whether the window title should be drawn even if the window
    * contents extend into the titlebar. Ignored on windows that don't draw
-   * in the titlebar. Only implemented on OS X.
+   * in the titlebar. Only implemented on macOS.
    */
   virtual void SetDrawsTitle(bool aDrawTitle) {}
 
-  /**
-   * Indicates whether the widget should attempt to make titlebar controls
-   * easier to see on dark titlebar backgrounds.
-   */
-  virtual void SetUseBrightTitlebarForeground(bool aBrightForeground) {}
-
   /**
    * Hide window chrome (borders, buttons) for this widget.
    *
@@ -1224,27 +1165,21 @@ class nsIWidget : public nsISupports {
 
   /**
    * Put the toplevel window into or out of fullscreen mode.
-   * If aTargetScreen is given, attempt to go fullscreen on that screen,
-   * if possible.  (If not, it behaves as if aTargetScreen is null.)
-   * If !aFullScreen, aTargetScreen is ignored.
-   * aTargetScreen support is currently only implemented on Windows.
    *
    * @return NS_OK if the widget is setup properly for fullscreen and
    * FullscreenChanged callback has been or will be called. If other
    * value is returned, the caller should continue the change itself.
    */
-  virtual nsresult MakeFullScreen(bool aFullScreen,
-                                  nsIScreen* aTargetScreen = nullptr) = 0;
+  virtual nsresult MakeFullScreen(bool aFullScreen) = 0;
 
   /**
    * Same as MakeFullScreen, except that, on systems which natively
    * support fullscreen transition, calling this method explicitly
    * requests that behavior.
-   * It is currently only supported on OS X 10.7+.
+   * It is currently only supported on macOS 10.7+.
    */
-  virtual nsresult MakeFullScreenWithNativeTransition(
-      bool aFullScreen, nsIScreen* aTargetScreen = nullptr) {
-    return MakeFullScreen(aFullScreen, aTargetScreen);
+  virtual nsresult MakeFullScreenWithNativeTransition(bool aFullScreen) {
+    return MakeFullScreen(aFullScreen);
   }
 
   /**
@@ -1262,24 +1197,7 @@ class nsIWidget : public nsISupports {
    * Return the widget's LayerManager. The layer tree for that
    * LayerManager is what gets rendered to the widget.
    */
-  inline LayerManager* GetLayerManager() {
-    return GetLayerManager(nullptr, mozilla::layers::LayersBackend::LAYERS_NONE,
-                           LAYER_MANAGER_CURRENT);
-  }
-
-  inline LayerManager* GetLayerManager(LayerManagerPersistence aPersistence) {
-    return GetLayerManager(nullptr, mozilla::layers::LayersBackend::LAYERS_NONE,
-                           aPersistence);
-  }
-
-  /**
-   * Like GetLayerManager(), but prefers creating a layer manager of
-   * type |aBackendHint| instead of what would normally be created.
-   * LayersBackend::LAYERS_NONE means "no hint".
-   */
-  virtual LayerManager* GetLayerManager(
-      PLayerTransactionChild* aShadowManager, LayersBackend aBackendHint,
-      LayerManagerPersistence aPersistence = LAYER_MANAGER_CURRENT) = 0;
+  virtual WindowRenderer* GetWindowRenderer() = 0;
 
   /**
    * Called before each layer manager transaction to allow any preparation
@@ -1389,6 +1307,12 @@ class nsIWidget : public nsISupports {
     return mozilla::LayoutDeviceToLayoutDeviceMatrix4x4();
   }
 
+  mozilla::LayoutDeviceIntPoint WidgetToTopLevelWidgetOffset() {
+    return mozilla::LayoutDeviceIntPoint::Round(
+        WidgetToTopLevelWidgetTransform().TransformPoint(
+            mozilla::LayoutDevicePoint()));
+  }
+
   /**
    * Given the specified client size, return the corresponding window size,
    * which includes the area for the borders and titlebar. This method
@@ -1409,12 +1333,27 @@ class nsIWidget : public nsISupports {
    */
   virtual void DispatchEventToAPZOnly(mozilla::WidgetInputEvent* aEvent) = 0;
 
+  /*
+   * Dispatch a gecko event for this widget.
+   * Returns true if it's consumed.  Otherwise, false.
+   */
+  virtual bool DispatchWindowEvent(mozilla::WidgetGUIEvent& event) = 0;
+
+  // A structure that groups the statuses from APZ dispatch and content
+  // dispatch.
+  struct ContentAndAPZEventStatus {
+    // Either of these may not be set if the event was not dispatched
+    // to APZ or to content.
+    nsEventStatus mApzStatus = nsEventStatus_eIgnore;
+    nsEventStatus mContentStatus = nsEventStatus_eIgnore;
+  };
+
   /**
    * Dispatches an event that must be handled by APZ first, when APZ is
    * enabled. If invoked in the child process, it is forwarded to the
    * parent process synchronously.
    */
-  virtual nsEventStatus DispatchInputEvent(
+  virtual ContentAndAPZEventStatus DispatchInputEvent(
       mozilla::WidgetInputEvent* aEvent) = 0;
 
   /**
@@ -1430,6 +1369,10 @@ class nsIWidget : public nsISupports {
    */
   virtual bool AsyncPanZoomEnabled() const = 0;
 
+  /**
+   */
+  virtual void SwipeFinished() = 0;
+
   /**
    * Enables the dropping of files to a widget.
    */
@@ -1508,7 +1451,10 @@ class nsIWidget : public nsISupports {
       mozilla::WidgetGUIEvent* aEvent, int32_t aHorizontal,
       int32_t aVertical) = 0;
 
-  enum Modifiers {
+  // TODO: Make this an enum class with MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS or
+  //       EnumSet class.
+  enum Modifiers : uint32_t {
+    NO_MODIFIERS = 0x00000000,
     CAPS_LOCK = 0x00000001,  // when CapsLock is active
     NUM_LOCK = 0x00000002,   // when NumLock is active
     SHIFT_L = 0x00000100,
@@ -1566,17 +1512,25 @@ class nsIWidget : public nsISupports {
    * z-order.
    * @param aPoint screen location of the mouse, in device
    * pixels, with origin at the top left
-   * @param aNativeMessage *platform-specific* event type (e.g. on Mac,
-   * NSMouseMoved; on Windows, MOUSEEVENTF_MOVE, MOUSEEVENTF_LEFTDOWN etc)
-   * @param aModifierFlags *platform-specific* modifier flags (ignored
-   * on Windows)
+   * @param aNativeMessage abstract native message.
+   * @param aButton Mouse button defined by DOM UI Events.
+   * @param aModifierFlags Some values of nsIWidget::Modifiers.
+   *                       FYI: On Windows, Android and Headless widget on all
+   *                       platroms, this hasn't been handled yet.
    * @param aObserver the observer that will get notified once the events
    * have been dispatched.
    */
-  virtual nsresult SynthesizeNativeMouseEvent(LayoutDeviceIntPoint aPoint,
-                                              uint32_t aNativeMessage,
-                                              uint32_t aModifierFlags,
-                                              nsIObserver* aObserver) = 0;
+  enum class NativeMouseMessage : uint32_t {
+    ButtonDown,   // button down
+    ButtonUp,     // button up
+    Move,         // mouse cursor move
+    EnterWindow,  // mouse cursor comes into a window
+    LeaveWindow,  // mouse cursor leaves from a window
+  };
+  virtual nsresult SynthesizeNativeMouseEvent(
+      LayoutDeviceIntPoint aPoint, NativeMouseMessage aNativeMessage,
+      mozilla::MouseButton aButton, nsIWidget::Modifiers aModifierFlags,
+      nsIObserver* aObserver) = 0;
 
   /**
    * A shortcut to SynthesizeNativeMouseEvent, abstracting away the native
@@ -1636,7 +1590,15 @@ class nsIWidget : public nsISupports {
     // ALL_BITS used for validity checking during IPC serialization
     ALL_BITS = (1 << 4) - 1
   };
-
+  /*
+   * TouchpadGesturePhase states for SynthesizeNativeTouchPadPinch and
+   * SynthesizeNativeTouchpadPan. Match phase states in nsIDOMWindowUtils.idl.
+   */
+  enum TouchpadGesturePhase {
+    PHASE_BEGIN = 0,
+    PHASE_UPDATE = 1,
+    PHASE_END = 2
+  };
   /*
    * Create a new or update an existing touch pointer on the digitizer.
    * To trigger os level gestures, individual touch points should
@@ -1658,6 +1620,12 @@ class nsIWidget : public nsISupports {
                                               double aPointerPressure,
                                               uint32_t aPointerOrientation,
                                               nsIObserver* aObserver) = 0;
+  /*
+   * See nsIDOMWindowUtils.sendNativeTouchpadPinch().
+   */
+  virtual nsresult SynthesizeNativeTouchPadPinch(
+      TouchpadGesturePhase aEventPhase, float aScale,
+      LayoutDeviceIntPoint aPoint, int32_t aModifierFlags) = 0;
 
   /*
    * Helper for simulating a simple tap event with one touch point. When
@@ -1671,6 +1639,14 @@ class nsIWidget : public nsISupports {
                                             bool aLongTap,
                                             nsIObserver* aObserver);
 
+  virtual nsresult SynthesizeNativePenInput(uint32_t aPointerId,
+                                            TouchPointerState aPointerState,
+                                            LayoutDeviceIntPoint aPoint,
+                                            double aPressure,
+                                            uint32_t aRotation, int32_t aTiltX,
+                                            int32_t aTiltY, int32_t aButton,
+                                            nsIObserver* aObserver) = 0;
+
   /*
    * Cancels all active simulated touch input points and pending long taps.
    * Native widgets should track existing points such that they can clear the
@@ -1680,6 +1656,22 @@ class nsIWidget : public nsISupports {
    */
   virtual nsresult ClearNativeTouchSequence(nsIObserver* aObserver);
 
+  /*
+   * Send a native event as if the user double tapped the touchpad with two
+   * fingers.
+   */
+  virtual nsresult SynthesizeNativeTouchpadDoubleTap(
+      LayoutDeviceIntPoint aPoint, uint32_t aModifierFlags) = 0;
+
+  /*
+   * See nsIDOMWindowUtils.sendNativeTouchpadPan().
+   */
+  virtual nsresult SynthesizeNativeTouchpadPan(TouchpadGesturePhase aEventPhase,
+                                               LayoutDeviceIntPoint aPoint,
+                                               double aDeltaX, double aDeltaY,
+                                               int32_t aModifierFlags,
+                                               nsIObserver* aObserver) = 0;
+
   virtual void StartAsyncScrollbarDrag(
       const AsyncDragMetrics& aDragMetrics) = 0;
 
@@ -1698,6 +1690,8 @@ class nsIWidget : public nsISupports {
    */
   virtual void StopAsyncAutoscroll(const ScrollableLayerGuid& aGuid) = 0;
 
+  virtual LayersId GetRootLayerTreeId() = 0;
+
   // If this widget supports out-of-process compositing, it can override
   // this method to provide additional information to the compositor.
   virtual void GetCompositorWidgetInitData(
@@ -1713,23 +1707,23 @@ class nsIWidget : public nsISupports {
     return NS_ERROR_NOT_IMPLEMENTED;
   }
 
-  // Get rectangle of the screen where the window is placed.
-  // It's used to detect popup overflow under Wayland because
-  // Screenmanager does not work under it.
-#ifdef MOZ_WAYLAND
-  virtual nsresult GetScreenRect(LayoutDeviceIntRect* aRect) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-  virtual nsRect GetPreferredPopupRect() {
-    NS_WARNING("GetPreferredPopupRect implemented only for wayland");
-    return nsRect(0, 0, 0, 0);
-  }
-  virtual void FlushPreferredPopupRect() {
-    NS_WARNING("FlushPreferredPopupRect implemented only for wayland");
-    return;
+  /**
+   * Wayland specific routines.
+   */
+  virtual LayoutDeviceIntSize GetMoveToRectPopupSize() const {
+    NS_WARNING("GetLayoutPopupRect implemented only for wayland");
+    return LayoutDeviceIntSize();
   }
 
-#endif
+  /**
+   * If this widget uses native pointer lock instead of warp-to-center
+   * (currently only GTK on Wayland), these methods provide access to that
+   * functionality.
+   */
+  virtual void SetNativePointerLockCenter(
+      const LayoutDeviceIntPoint& aLockCenter) {}
+  virtual void LockNativePointer() {}
+  virtual void UnlockNativePointer() {}
 
   /*
    * Get safe area insets except to cutout.
@@ -1825,50 +1819,13 @@ class nsIWidget : public nsISupports {
   virtual nsresult NotifyIME(const IMENotification& aIMENotification) = 0;
 
   /**
-   * Start plugin IME.  If this results in a string getting committed, the
-   * result is in aCommitted (otherwise aCommitted is empty).
-   *
-   * aKeyboardEvent     The event with which plugin IME is to be started
-   * panelX and panelY  Location in screen coordinates of the IME input panel
-   *                    (should be just under the plugin)
-   * aCommitted         The string committed during IME -- otherwise empty
+   * MaybeDispatchInitialFocusEvent will dispatch a focus event after creation
+   * of the widget, in the event that we were not able to observe and respond to
+   * the initial focus event. This is necessary for the early skeleton UI
+   * window, which is displayed and receives its initial focus event before we
+   * can actually respond to it.
    */
-  [[nodiscard]] virtual nsresult StartPluginIME(
-      const mozilla::WidgetKeyboardEvent& aKeyboardEvent, int32_t aPanelX,
-      int32_t aPanelY, nsString& aCommitted) = 0;
-
-  /**
-   * Tells the widget whether or not a plugin (inside the widget) has the
-   * keyboard focus.  Should be sent when the keyboard focus changes too or
-   * from a plugin.
-   *
-   * aFocused  Whether or not a plugin is focused
-   */
-  virtual void SetPluginFocused(bool& aFocused) = 0;
-
-  /*
-   * Tell the plugin has focus.  It is unnecessary to use IPC
-   */
-  bool PluginHasFocus() {
-    return GetInputContext().mIMEState.mEnabled == IMEState::PLUGIN;
-  }
-
-  /**
-   * Set IME candidate window position by windowless plugin.
-   */
-  virtual void SetCandidateWindowForPlugin(
-      const mozilla::widget::CandidateWindowPosition& aPosition) = 0;
-
-  /**
-   * Handle default action when PluginEvent isn't handled
-   */
-  virtual void DefaultProcOfPluginEvent(
-      const mozilla::WidgetPluginEvent& aEvent) = 0;
-
-  /*
-   * Enable or Disable IME by windowless plugin.
-   */
-  virtual void EnableIMEForPlugin(bool aEnable) {}
+  virtual void MaybeDispatchInitialFocusEvent() {}
 
   /*
    * Notifies the input context changes.
@@ -1901,14 +1858,10 @@ class nsIWidget : public nsISupports {
    * Retrieve edit commands when the key combination of aEvent is used
    * in platform native applications.
    */
-  enum NativeKeyBindingsType : uint8_t {
-    NativeKeyBindingsForSingleLineEditor,
-    NativeKeyBindingsForMultiLineEditor,
-    NativeKeyBindingsForRichTextEditor
-  };
-  virtual bool GetEditCommands(NativeKeyBindingsType aType,
-                               const mozilla::WidgetKeyboardEvent& aEvent,
-                               nsTArray<mozilla::CommandInt>& aCommands);
+  MOZ_CAN_RUN_SCRIPT virtual bool GetEditCommands(
+      mozilla::NativeKeyBindingsType aType,
+      const mozilla::WidgetKeyboardEvent& aEvent,
+      nsTArray<mozilla::CommandInt>& aCommands);
 
   /*
    * Retrieves a reference to notification requests of IME.  Note that the
@@ -1952,15 +1905,6 @@ class nsIWidget : public nsISupports {
 
   static already_AddRefed<nsIWidget> CreateHeadlessWidget();
 
-  /**
-   * Allocate and return a "plugin proxy widget", a subclass of PuppetWidget
-   * used in wrapping a PPluginWidget connection for remote widgets. Note
-   * this call creates the base object, it does not create the widget. Use
-   * nsIWidget's Create to do this.
-   */
-  static already_AddRefed<nsIWidget> CreatePluginProxyWidget(
-      BrowserChild* aBrowserChild, mozilla::plugins::PluginWidgetChild* aActor);
-
   /**
    * Reparent this widget's native widget.
    * @param aNewParent the native widget of aNewParent is the new native
@@ -2016,6 +1960,14 @@ class nsIWidget : public nsISupports {
    */
   virtual const SizeConstraints GetSizeConstraints() = 0;
 
+  /**
+   * Apply the current size constraints to the given size.
+   *
+   * @param aWidth width to constrain
+   * @param aHeight height to constrain
+   */
+  virtual void ConstrainSize(int32_t* aWidth, int32_t* aHeight) = 0;
+
   /**
    * If this is owned by a BrowserChild, return that.  Otherwise return
    * null.
@@ -2034,10 +1986,17 @@ class nsIWidget : public nsISupports {
   virtual void ClearCachedWebrenderResources() {}
 
   /**
-   * If this widget has its own vsync source, return it, otherwise return
-   * nullptr. An example of such local source would be Wayland frame callbacks.
+   * Request fast snapshot at RenderCompositor of WebRender.
+   * Since readback of Windows DirectComposition is very slow.
    */
-  virtual RefPtr<mozilla::gfx::VsyncSource> GetVsyncSource() { return nullptr; }
+  virtual bool SetNeedFastSnaphot() { return false; }
+
+  /**
+   * If this widget has its own vsync dispatcher, return it, otherwise return
+   * nullptr. An example of such a local vsync dispatcher would be Wayland frame
+   * callbacks.
+   */
+  virtual RefPtr<mozilla::VsyncDispatcher> GetVsyncDispatcher();
 
   /**
    * Returns true if the widget requires synchronous repaints on resize,
@@ -2074,28 +2033,6 @@ class nsIWidget : public nsISupports {
                           const ScrollableLayerGuid::ViewID& aViewId,
                           const CSSRect& aRect, const uint32_t& aFlags) = 0;
 
-  /**
-   * OnWindowedPluginKeyEvent() is called when native key event is
-   * received in the focused plugin process directly in PluginInstanceChild.
-   *
-   * @param aKeyEventData     The native key event data.  The actual type
-   *                          copied into NativeEventData depends on the
-   *                          caller.  Please check PluginInstanceChild.
-   * @param aCallback         Callback interface.  When this returns
-   *                          NS_SUCCESS_EVENT_HANDLED_ASYNCHRONOUSLY,
-   *                          the event handler has to call this callback.
-   *                          Otherwise, the caller should do that instead.
-   * @return                  NS_ERROR_* if this fails to handle the event.
-   *                          NS_SUCCESS_EVENT_CONSUMED if the key event is
-   *                          consumed.
-   *                          NS_OK if the key event isn't consumed.
-   *                          NS_SUCCESS_EVENT_HANDLED_ASYNCHRONOUSLY if the
-   *                          key event will be handled asynchronously.
-   */
-  virtual nsresult OnWindowedPluginKeyEvent(
-      const mozilla::NativeEventData& aKeyEventData,
-      nsIKeyEventInPluginCallback* aCallback);
-
   /**
    * LookUpDictionary shows the dictionary for the word around current point.
    *
@@ -2140,7 +2077,8 @@ class nsIWidget : public nsISupports {
    * @param aSize size of the buffer in screen pixels.
    */
   virtual void RecvScreenPixels(mozilla::ipc::Shmem&& aMem,
-                                const ScreenIntSize& aSize) = 0;
+                                const ScreenIntSize& aSize,
+                                bool aNeedsYFlip) = 0;
 
   virtual void UpdateDynamicToolbarMaxHeight(mozilla::ScreenIntCoord aHeight) {}
   virtual mozilla::ScreenIntCoord GetDynamicToolbarMaxHeight() const {
@@ -2156,6 +2094,15 @@ class nsIWidget : public nsISupports {
    */
   virtual double GetDefaultScaleInternal() { return 1.0; }
 
+  /**
+   * Layout uses this to alert the widget to the client rect representing
+   * the window maximize button.  An empty rect indicates there is no
+   * maximize button (for example, in fullscreen).  This is only implemented
+   * on Windows.
+   */
+  virtual void SetWindowButtonRect(WindowButtonType aButtonType,
+                                   const LayoutDeviceIntRect& aClientRect) {}
+
  protected:
   // keep the list of children.  We also keep track of our siblings.
   // The ownership model is as follows: parent holds a strong ref to
