@@ -44,13 +44,7 @@ TaskbarPreview::TaskbarPreview(ITaskbarList4* aTaskbar,
       mController(aController),
       mWnd(aHWND),
       mVisible(false),
-      mDocShell(do_GetWeakReference(aShell)) {
-  // TaskbarPreview may outlive the WinTaskbar that created it
-  ::CoInitialize(nullptr);
-
-  WindowHook& hook = GetWindowHook();
-  hook.AddMonitor(WM_DESTROY, MainWindowHook, this);
-}
+      mDocShell(do_GetWeakReference(aShell)) {}
 
 TaskbarPreview::~TaskbarPreview() {
   // Avoid dangling pointer
@@ -67,6 +61,19 @@ TaskbarPreview::~TaskbarPreview() {
   ::CoUninitialize();
 }
 
+nsresult TaskbarPreview::Init() {
+  // TaskbarPreview may outlive the WinTaskbar that created it
+  if (FAILED(::CoInitialize(nullptr))) {
+    return NS_ERROR_NOT_INITIALIZED;
+  }
+
+  WindowHook* hook = GetWindowHook();
+  if (!hook) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  return hook->AddMonitor(WM_DESTROY, MainWindowHook, this);
+}
+
 NS_IMETHODIMP
 TaskbarPreview::SetController(nsITaskbarPreviewController* aController) {
   NS_ENSURE_ARG(aController);
@@ -163,9 +170,11 @@ nsresult TaskbarPreview::Enable() {
   if (CanMakeTaskbarCalls()) {
     rv = UpdateTaskbarProperties();
   } else if (IsWindowAvailable()) {
-    WindowHook& hook = GetWindowHook();
-    hook.AddMonitor(nsAppShell::GetTaskbarButtonCreatedMessage(),
-                    MainWindowHook, this);
+    WindowHook* hook = GetWindowHook();
+    MOZ_ASSERT(hook,
+               "IsWindowAvailable() should have eliminated the null case.");
+    hook->AddMonitor(nsAppShell::GetTaskbarButtonCreatedMessage(),
+                     MainWindowHook, this);
   }
   return rv;
 }
@@ -176,9 +185,10 @@ nsresult TaskbarPreview::Disable() {
     return NS_OK;
   }
 
-  WindowHook& hook = GetWindowHook();
-  (void)hook.RemoveMonitor(nsAppShell::GetTaskbarButtonCreatedMessage(),
-                           MainWindowHook, this);
+  WindowHook* hook = GetWindowHook();
+  MOZ_ASSERT(hook, "IsWindowAvailable() should have eliminated the null case.");
+  (void)hook->RemoveMonitor(nsAppShell::GetTaskbarButtonCreatedMessage(),
+                            MainWindowHook, this);
 
   return NS_OK;
 }
@@ -194,8 +204,9 @@ bool TaskbarPreview::IsWindowAvailable() const {
 }
 
 void TaskbarPreview::DetachFromNSWindow() {
-  WindowHook& hook = GetWindowHook();
-  hook.RemoveMonitor(WM_DESTROY, MainWindowHook, this);
+  if (WindowHook* hook = GetWindowHook()) {
+    hook->RemoveMonitor(WM_DESTROY, MainWindowHook, this);
+  }
   mWnd = nullptr;
 }
 
@@ -252,16 +263,16 @@ bool TaskbarPreview::CanMakeTaskbarCalls() {
   if (mVisible) {
     nsWindow* window = WinUtils::GetNSWindowPtr(mWnd);
     NS_ASSERTION(window, "Could not get nsWindow from HWND");
-    return window->HasTaskbarIconBeenCreated();
+    return window ? window->HasTaskbarIconBeenCreated() : false;
   }
   return false;
 }
 
-WindowHook& TaskbarPreview::GetWindowHook() {
+WindowHook* TaskbarPreview::GetWindowHook() {
   nsWindow* window = WinUtils::GetNSWindowPtr(mWnd);
   NS_ASSERTION(window, "Cannot use taskbar previews in an embedded context!");
 
-  return window->GetWindowHook();
+  return window ? &window->GetWindowHook() : nullptr;
 }
 
 void TaskbarPreview::EnableCustomDrawing(HWND aHWND, bool aEnable) {
@@ -364,6 +375,7 @@ TaskbarPreviewCallback::Done(nsISupports* aCanvas, bool aDrawBorder) {
     hr = DwmSetIconicThumbnail(mPreview->PreviewWindow(), hBitmap, flags);
   }
   MOZ_ASSERT(SUCCEEDED(hr));
+  (void)hr;
   return NS_OK;
 }
 
