@@ -7,7 +7,7 @@
 #define WinPointerEvents_h__
 
 #include "mozilla/MouseEvents.h"
-#include "nsWindowBase.h"
+#include "nsWindow.h"
 
 // Define PointerEvent related macros and structures when building code on
 // Windows version before Win8.
