@@ -9,6 +9,7 @@
 #include <ole2.h>
 #include <shlobj.h>
 
+#include "nsComponentManagerUtils.h"
 #include "nsDataObj.h"
 #include "nsArrayUtils.h"
 #include "nsClipboard.h"
@@ -29,6 +30,7 @@
 #include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/Unused.h"
 #include "nsProxyRelease.h"
+#include "nsIObserverService.h"
 #include "nsIOutputStream.h"
 #include "nscore.h"
 #include "nsDirectoryServiceDefs.h"
@@ -39,8 +41,11 @@
 #include "nsIPrincipal.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsMimeTypes.h"
+#include "nsIMIMEService.h"
 #include "imgIEncoder.h"
 #include "imgITools.h"
+#include "WinUtils.h"
+#include "nsLocalFile.h"
 
 #include "mozilla/LazyIdleThread.h"
 #include <algorithm>
@@ -158,7 +163,8 @@ NS_IMETHODIMP nsDataObj::CStream::OnStopRequest(nsIRequest* aRequest,
 // and cancel the operation.
 nsresult nsDataObj::CStream::WaitForCompletion() {
   // We are guaranteed OnStopRequest will get called, so this should be ok.
-  SpinEventLoopUntil([&]() { return mChannelRead; });
+  SpinEventLoopUntil("widget:nsDataObj::CStream::WaitForCompletion"_ns,
+                     [&]() { return mChannelRead; });
 
   if (!mChannelData.Length()) mChannelResult = NS_ERROR_FAILURE;
 
@@ -463,14 +469,6 @@ STDMETHODIMP nsDataObj::CMemStream::Stat(STATSTG* statstg, DWORD dwFlags) {
   return S_OK;
 }
 
-/*
- * deliberately not using MAX_PATH. This is because on platforms < XP
- * a file created with a long filename may be mishandled by the shell
- * resulting in it not being able to be deleted or moved.
- * See bug 250392 for more details.
- */
-#define NS_MAX_FILEDESCRIPTOR 128 + 1
-
 /*
  * Class nsDataObj
  */
@@ -545,7 +543,7 @@ STDMETHODIMP_(ULONG) nsDataObj::AddRef() {
 }
 
 namespace {
-class RemoveTempFileHelper final : public nsIObserver {
+class RemoveTempFileHelper final : public nsIObserver, public nsINamed {
  public:
   explicit RemoveTempFileHelper(nsIFile* aTempFile) : mTempFile(aTempFile) {
     MOZ_ASSERT(mTempFile);
@@ -575,6 +573,7 @@ class RemoveTempFileHelper final : public nsIObserver {
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIOBSERVER
+  NS_DECL_NSINAMED
 
  private:
   ~RemoveTempFileHelper() {
@@ -587,7 +586,7 @@ class RemoveTempFileHelper final : public nsIObserver {
   nsCOMPtr<nsITimer> mTimer;
 };
 
-NS_IMPL_ISUPPORTS(RemoveTempFileHelper, nsIObserver);
+NS_IMPL_ISUPPORTS(RemoveTempFileHelper, nsIObserver, nsINamed);
 
 NS_IMETHODIMP
 RemoveTempFileHelper::Observe(nsISupports* aSubject, const char* aTopic,
@@ -614,6 +613,12 @@ RemoveTempFileHelper::Observe(nsISupports* aSubject, const char* aTopic,
   }
   return NS_OK;
 }
+
+NS_IMETHODIMP
+RemoveTempFileHelper::GetName(nsACString& aName) {
+  aName.AssignLiteral("RemoveTempFileHelper");
+  return NS_OK;
+}
 }  // namespace
 
 //-----------------------------------------------------
@@ -1076,88 +1081,65 @@ nsDataObj ::GetFileContents(FORMATETC& aFE, STGMEDIUM& aSTG) {
 
 }  // GetFileContents
 
-//
-// Given a unicode string, we ensure that it contains only characters which are
-// valid within the file system. Remove all forbidden characters from the name,
-// and completely disallow any title that starts with a forbidden name and
-// extension (e.g. "nul" is invalid, but "nul." and "nul.txt" are also invalid
-// and will cause problems).
-//
-// It would seem that this is more functionality suited to being in nsIFile.
-//
-static void MangleTextToValidFilename(nsString& aText) {
-  static const char* forbiddenNames[] = {
-      "COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7",  "COM8",
-      "COM9", "LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6",  "LPT7",
-      "LPT8", "LPT9", "CON",  "PRN",  "AUX",  "NUL",  "CLOCK$"};
-
-  aText.StripChars(FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS);
-  aText.CompressWhitespace(true, true);
-  uint32_t nameLen;
-  for (size_t n = 0; n < ArrayLength(forbiddenNames); ++n) {
-    nameLen = (uint32_t)strlen(forbiddenNames[n]);
-    if (aText.EqualsIgnoreCase(forbiddenNames[n], nameLen)) {
-      // invalid name is either the entire string, or a prefix with a period
-      if (aText.Length() == nameLen || aText.CharAt(nameLen) == char16_t('.')) {
-        aText.Truncate();
-        break;
-      }
-    }
+// Ensure that the supplied name doesn't have invalid characters.
+static void ValidateFilename(nsString& aFilename, bool isShortcut) {
+  nsCOMPtr<nsIMIMEService> mimeService = do_GetService("@mozilla.org/mime;1");
+  if (NS_WARN_IF(!mimeService)) {
+    aFilename.Truncate();
+    return;
   }
+
+  uint32_t flags = nsIMIMEService::VALIDATE_SANITIZE_ONLY;
+  if (isShortcut) {
+    flags |= nsIMIMEService::VALIDATE_ALLOW_INVALID_FILENAMES;
+  }
+
+  nsAutoString outFilename;
+  mimeService->ValidateFileNameForSaving(aFilename, EmptyCString(), flags,
+                                         outFilename);
+  aFilename = outFilename;
 }
 
 //
-// Given a unicode string, convert it down to a valid local charset filename
-// with the supplied extension. This ensures that we do not cut MBCS characters
-// in the middle.
+// Given a unicode string, convert it to a valid local charset filename
+// and append the .url extension to be used for a shortcut file.
+// This ensures that we do not cut MBCS characters in the middle.
 //
 // It would seem that this is more functionality suited to being in nsIFile.
 //
-static bool CreateFilenameFromTextA(nsString& aText, const char* aExtension,
-                                    char* aFilename, uint32_t aFilenameLen) {
-  // ensure that the supplied name doesn't have invalid characters. If
-  // a valid mangled filename couldn't be created then it will leave the
-  // text empty.
-  MangleTextToValidFilename(aText);
-  if (aText.IsEmpty()) return false;
-
-  // repeatably call WideCharToMultiByte as long as the title doesn't fit in the
-  // buffer available to us. Continually reduce the length of the source title
-  // until the MBCS version will fit in the buffer with room for the supplied
-  // extension. Doing it this way ensures that even in MBCS environments there
-  // will be a valid MBCS filename of the correct length.
-  int maxUsableFilenameLen =
-      aFilenameLen - strlen(aExtension) - 1;  // space for ext + null byte
-  int currLen, textLen = (int)std::min(aText.Length(), aFilenameLen);
-  char defaultChar = '_';
-  do {
-    currLen = WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK | WC_DEFAULTCHAR,
-                                  aText.get(), textLen--, aFilename,
-                                  maxUsableFilenameLen, &defaultChar, nullptr);
-  } while (currLen == 0 && textLen > 0 &&
-           GetLastError() == ERROR_INSUFFICIENT_BUFFER);
-  if (currLen > 0 && textLen > 0) {
-    strcpy(&aFilename[currLen], aExtension);
-    return true;
-  } else {
-    // empty names aren't permitted
+static bool CreateURLFilenameFromTextA(nsAutoString& aText, char* aFilename) {
+  if (aText.IsEmpty()) {
     return false;
   }
+  aText.AppendLiteral(".URL");
+  ValidateFilename(aText, true);
+  if (aText.IsEmpty()) {
+    return false;
+  }
+
+  // ValidateFilename should already be checking the filename length, but do
+  // an extra check to verify for the local code page that the converted text
+  // doesn't go over MAX_PATH and just return false if it does.
+  char defaultChar = '_';
+  int currLen = WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK | WC_DEFAULTCHAR,
+                                    aText.get(), -1, aFilename, MAX_PATH,
+                                    &defaultChar, nullptr);
+  return currLen != 0;
 }
 
-static bool CreateFilenameFromTextW(nsString& aText, const wchar_t* aExtension,
-                                    wchar_t* aFilename, uint32_t aFilenameLen) {
-  // ensure that the supplied name doesn't have invalid characters. If
-  // a valid mangled filename couldn't be created then it will leave the
-  // text empty.
-  MangleTextToValidFilename(aText);
-  if (aText.IsEmpty()) return false;
+// Wide character version of CreateURLFilenameFromTextA
+static bool CreateURLFilenameFromTextW(nsAutoString& aText,
+                                       wchar_t* aFilename) {
+  if (aText.IsEmpty()) {
+    return false;
+  }
+  aText.AppendLiteral(".URL");
+  ValidateFilename(aText, true);
+  if (aText.IsEmpty() || aText.Length() >= MAX_PATH) {
+    return false;
+  }
 
-  const int extensionLen = wcslen(aExtension);
-  if (aText.Length() + extensionLen + 1 > aFilenameLen)
-    aText.Truncate(aFilenameLen - extensionLen - 1);
   wcscpy(&aFilename[0], aText.get());
-  wcscpy(&aFilename[aText.Length()], aExtension);
   return true;
 }
 
@@ -1204,13 +1186,11 @@ nsDataObj ::GetFileDescriptorInternetShortcutA(FORMATETC& aFE,
 
   // get a valid filename in the following order: 1) from the page title,
   // 2) localized string for an untitled page, 3) just use "Untitled.URL"
-  if (!CreateFilenameFromTextA(title, ".URL", fileGroupDescA->fgd[0].cFileName,
-                               NS_MAX_FILEDESCRIPTOR)) {
+  if (!CreateURLFilenameFromTextA(title, fileGroupDescA->fgd[0].cFileName)) {
     nsAutoString untitled;
     if (!GetLocalizedString("noPageTitle", untitled) ||
-        !CreateFilenameFromTextA(untitled, ".URL",
-                                 fileGroupDescA->fgd[0].cFileName,
-                                 NS_MAX_FILEDESCRIPTOR)) {
+        !CreateURLFilenameFromTextA(untitled,
+                                    fileGroupDescA->fgd[0].cFileName)) {
       strcpy(fileGroupDescA->fgd[0].cFileName, "Untitled.URL");
     }
   }
@@ -1247,13 +1227,11 @@ nsDataObj ::GetFileDescriptorInternetShortcutW(FORMATETC& aFE,
 
   // get a valid filename in the following order: 1) from the page title,
   // 2) localized string for an untitled page, 3) just use "Untitled.URL"
-  if (!CreateFilenameFromTextW(title, L".URL", fileGroupDescW->fgd[0].cFileName,
-                               NS_MAX_FILEDESCRIPTOR)) {
+  if (!CreateURLFilenameFromTextW(title, fileGroupDescW->fgd[0].cFileName)) {
     nsAutoString untitled;
     if (!GetLocalizedString("noPageTitle", untitled) ||
-        !CreateFilenameFromTextW(untitled, L".URL",
-                                 fileGroupDescW->fgd[0].cFileName,
-                                 NS_MAX_FILEDESCRIPTOR)) {
+        !CreateURLFilenameFromTextW(untitled,
+                                    fileGroupDescW->fgd[0].cFileName)) {
       wcscpy(fileGroupDescW->fgd[0].cFileName, L"Untitled.URL");
     }
   }
@@ -1960,8 +1938,8 @@ nsresult nsDataObj ::ExtractShortcutTitle(nsString& outTitle) {
 // BuildPlatformHTML
 //
 // Munge our HTML data to win32's CF_HTML spec. Basically, put the requisite
-// header information on it. This will null terminate |outPlatformHTML|. See
-//  http://msdn.microsoft.com/workshop/networking/clipboard/htmlclipboard.asp
+// header information on it. This will null-terminate |outPlatformHTML|. See
+//  https://docs.microsoft.com/en-us/windows/win32/dataxchg/html-clipboard-format
 // for details.
 //
 // We assume that |inOurHTML| is already a fragment (ie, doesn't have <HTML>
@@ -1971,15 +1949,7 @@ nsresult nsDataObj ::ExtractShortcutTitle(nsString& outTitle) {
 nsresult nsDataObj ::BuildPlatformHTML(const char* inOurHTML,
                                        char** outPlatformHTML) {
   *outPlatformHTML = nullptr;
-
   nsDependentCString inHTMLString(inOurHTML);
-  const char* const numPlaceholder = "00000000";
-  const char* const startHTMLPrefix = "Version:0.9\r\nStartHTML:";
-  const char* const endHTMLPrefix = "\r\nEndHTML:";
-  const char* const startFragPrefix = "\r\nStartFragment:";
-  const char* const endFragPrefix = "\r\nEndFragment:";
-  const char* const startSourceURLPrefix = "\r\nSourceURL:";
-  const char* const endFragTrailer = "\r\n";
 
   // Do we already have mSourceURL from a drag?
   if (mSourceURL.IsEmpty()) {
@@ -1989,60 +1959,84 @@ nsresult nsDataObj ::BuildPlatformHTML(const char* inOurHTML,
     AppendUTF16toUTF8(url, mSourceURL);
   }
 
-  const int32_t kSourceURLLength = mSourceURL.Length();
-  const int32_t kNumberLength = strlen(numPlaceholder);
+  constexpr auto kStartHTMLPrefix = "Version:0.9\r\nStartHTML:"_ns;
+  constexpr auto kEndHTMLPrefix = "\r\nEndHTML:"_ns;
+  constexpr auto kStartFragPrefix = "\r\nStartFragment:"_ns;
+  constexpr auto kEndFragPrefix = "\r\nEndFragment:"_ns;
+  constexpr auto kStartSourceURLPrefix = "\r\nSourceURL:"_ns;
+  constexpr auto kEndFragTrailer = "\r\n"_ns;
 
-  const int32_t kTotalHeaderLen =
-      strlen(startHTMLPrefix) + strlen(endHTMLPrefix) +
-      strlen(startFragPrefix) + strlen(endFragPrefix) + strlen(endFragTrailer) +
-      (kSourceURLLength > 0 ? strlen(startSourceURLPrefix) : 0) +
-      kSourceURLLength + (4 * kNumberLength);
+  // The CF_HTML's size is embedded in the fragment, in such a way that the
+  // number of digits in the size is part of the size itself. While it _is_
+  // technically possible to compute the size of the size-field precisely -- by
+  // trial and error, if nothing else -- it's simpler to just fix it at eight
+  // characters and zero-pad it. (Zero-padding is explicitly permitted by the
+  // format definition.)
+  //
+  // Of course, while a maximum size of (10**9 - 1) bytes would probably have
+  // covered all possible use-cases in 2001, it's somewhat more likely to happen
+  // nowadays. Bug 1754803 covers extending this code to handle gigabyte-sized
+  // copies.
+  constexpr size_t kNumberLength = 8;
 
-  constexpr auto htmlHeaderString = "<html><body>\r\n"_ns;
+  const size_t sourceURLLength = mSourceURL.Length();
 
-  constexpr auto fragmentHeaderString = "<!--StartFragment-->"_ns;
+  constexpr size_t kFixedHeaderLen =
+      kStartHTMLPrefix.Length() + kEndHTMLPrefix.Length() +
+      kStartFragPrefix.Length() + kEndFragPrefix.Length() +
+      kEndFragTrailer.Length() + (4 * kNumberLength);
 
-  nsDependentCString trailingString(
+  const size_t totalHeaderLen =
+      kFixedHeaderLen + (sourceURLLength > 0
+                             ? kStartSourceURLPrefix.Length() + sourceURLLength
+                             : 0);
+
+  constexpr auto kHeaderString = "<html><body>\r\n<!--StartFragment-->"_ns;
+  constexpr auto kTrailingString =
       "<!--EndFragment-->\r\n"
       "</body>\r\n"
-      "</html>");
+      "</html>"_ns;
 
   // calculate the offsets
-  int32_t startHTMLOffset = kTotalHeaderLen;
-  int32_t startFragOffset = startHTMLOffset + htmlHeaderString.Length() +
-                            fragmentHeaderString.Length();
+  size_t startHTMLOffset = totalHeaderLen;
+  size_t startFragOffset = startHTMLOffset + kHeaderString.Length();
 
-  int32_t endFragOffset = startFragOffset + inHTMLString.Length();
-
-  int32_t endHTMLOffset = endFragOffset + trailingString.Length();
+  size_t endFragOffset = startFragOffset + inHTMLString.Length();
+  size_t endHTMLOffset = endFragOffset + kTrailingString.Length();
 
   // now build the final version
   nsCString clipboardString;
   clipboardString.SetCapacity(endHTMLOffset);
 
-  clipboardString.Append(startHTMLPrefix);
-  clipboardString.Append(nsPrintfCString("%08u", startHTMLOffset));
+  // These implicitly must match kNumberLength, above.
+  clipboardString.Append(kStartHTMLPrefix);
+  clipboardString.AppendPrintf("%08zu", startHTMLOffset);
 
-  clipboardString.Append(endHTMLPrefix);
-  clipboardString.Append(nsPrintfCString("%08u", endHTMLOffset));
+  clipboardString.Append(kEndHTMLPrefix);
+  clipboardString.AppendPrintf("%08zu", endHTMLOffset);
 
-  clipboardString.Append(startFragPrefix);
-  clipboardString.Append(nsPrintfCString("%08u", startFragOffset));
+  clipboardString.Append(kStartFragPrefix);
+  clipboardString.AppendPrintf("%08zu", startFragOffset);
 
-  clipboardString.Append(endFragPrefix);
-  clipboardString.Append(nsPrintfCString("%08u", endFragOffset));
+  clipboardString.Append(kEndFragPrefix);
+  clipboardString.AppendPrintf("%08zu", endFragOffset);
 
-  if (kSourceURLLength > 0) {
-    clipboardString.Append(startSourceURLPrefix);
+  if (sourceURLLength > 0) {
+    clipboardString.Append(kStartSourceURLPrefix);
     clipboardString.Append(mSourceURL);
   }
 
-  clipboardString.Append(endFragTrailer);
+  clipboardString.Append(kEndFragTrailer);
 
-  clipboardString.Append(htmlHeaderString);
-  clipboardString.Append(fragmentHeaderString);
+  // Assert that the positional values were correct as we pass by their
+  // corresponding positions.
+  MOZ_ASSERT(clipboardString.Length() == startHTMLOffset);
+  clipboardString.Append(kHeaderString);
+  MOZ_ASSERT(clipboardString.Length() == startFragOffset);
   clipboardString.Append(inHTMLString);
-  clipboardString.Append(trailingString);
+  MOZ_ASSERT(clipboardString.Length() == endFragOffset);
+  clipboardString.Append(kTrailingString);
+  MOZ_ASSERT(clipboardString.Length() == endHTMLOffset);
 
   *outPlatformHTML = ToNewCString(clipboardString, mozilla::fallible);
   if (!*outPlatformHTML) return NS_ERROR_OUT_OF_MEMORY;
@@ -2153,10 +2147,10 @@ HRESULT nsDataObj::GetDownloadDetails(nsIURI** aSourceURI,
     NS_UnescapeURL(urlFileName);
     CopyUTF8toUTF16(urlFileName, srcFileName);
   }
-  if (srcFileName.IsEmpty()) return E_FAIL;
 
   // make the name safe for the filesystem
-  MangleTextToValidFilename(srcFileName);
+  ValidateFilename(srcFileName, false);
+  if (srcFileName.IsEmpty()) return E_FAIL;
 
   sourceURI.swap(*aSourceURI);
   aFilename = srcFileName;
@@ -2187,9 +2181,8 @@ HRESULT nsDataObj::GetFileDescriptor_IStreamA(FORMATETC& aFE, STGMEDIUM& aSTG) {
   nsAutoCString nativeFileName;
   NS_CopyUnicodeToNative(wideFileName, nativeFileName);
 
-  strncpy(fileGroupDescA->fgd[0].cFileName, nativeFileName.get(),
-          NS_MAX_FILEDESCRIPTOR - 1);
-  fileGroupDescA->fgd[0].cFileName[NS_MAX_FILEDESCRIPTOR - 1] = '\0';
+  strncpy(fileGroupDescA->fgd[0].cFileName, nativeFileName.get(), MAX_PATH - 1);
+  fileGroupDescA->fgd[0].cFileName[MAX_PATH - 1] = '\0';
 
   // one file in the file block
   fileGroupDescA->cItems = 1;
@@ -2223,9 +2216,8 @@ HRESULT nsDataObj::GetFileDescriptor_IStreamW(FORMATETC& aFE, STGMEDIUM& aSTG) {
     return res;
   }
 
-  wcsncpy(fileGroupDescW->fgd[0].cFileName, wideFileName.get(),
-          NS_MAX_FILEDESCRIPTOR - 1);
-  fileGroupDescW->fgd[0].cFileName[NS_MAX_FILEDESCRIPTOR - 1] = '\0';
+  wcsncpy(fileGroupDescW->fgd[0].cFileName, wideFileName.get(), MAX_PATH - 1);
+  fileGroupDescW->fgd[0].cFileName[MAX_PATH - 1] = '\0';
   // one file in the file block
   fileGroupDescW->cItems = 1;
   fileGroupDescW->fgd[0].dwFlags = FD_PROGRESSUI;
